/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file  InputManager.cpp
\author Zach Dolph
\par E-mail: z.dolph@digipen.edu
\par Project: Mecha Engine
\date 09/30/2016
\brief


\par
  Functions Include:
  -
*/
/*****************************************************************************/
#include <iostream>
#include "Context.h"
#include "StateFactory.h"
#include "SpaceManager.h"
#include "AudioEngine.h"

#include "InputManager.h"

InputManager* InputManager::s_pInstance = 0;


InputManager::InputManager() : 
_down(), _pressed(),
m_bJoysticksInitialized(false), m_mousePosition(new glm::vec2(0, 0)), 
keyPressed(false), _mouseButtonDown(), _mouseButtonPressed()
{
}

InputManager::~InputManager()
{
  // free the allocated memory
  delete m_mousePosition;

  // clear the arrays used to store info
  m_joystickValues.clear();
  m_triggerValues.clear();
  m_joysticks.clear();
  m_buttonStates.clear();
  m_dpadStates.clear();

}

void InputManager::removeJoystick(SDL_Event& event)
{
  SDL_Joystick* joy = SDL_JoystickOpen(event.jdevice.which);
  SDL_JoystickClose(joy);
  m_joystickValues.pop_back();
  m_joysticks.pop_back();
}

bool InputManager::joysticksInitialized()
{
  return m_bJoysticksInitialized;
}

void InputManager::initializeJoysticks()
{
  // if we havent initialized the joystick subsystem then do it here
  if (SDL_WasInit(SDL_INIT_JOYSTICK) == 0)
  {
    SDL_InitSubSystem(SDL_INIT_JOYSTICK);
  }

  // get the number of joysticks, skip init if there arent any yo
  if (SDL_NumJoysticks() > 0)
  {
    for (int i = 0; i < SDL_NumJoysticks(); i++)
    {
      // create the new joystick
      SDL_Joystick* joy = SDL_JoystickOpen(i);

      // if the joystick opened correctly, than we need to populate our arrays
      if (SDL_JoystickOpen(i))
      {
        // push back into the array to be closed later
        m_joysticks.push_back(joy);

        // create a pair of values for the axis, a vector assigned to each stick yo
        m_joystickValues.push_back(std::make_pair(new glm::vec2(0, 0), new glm::vec2(0, 0)));
        m_triggerValues.push_back(std::make_pair(new glm::vec2(0, 0), new glm::vec2(0, 0)));

        // create an array to hold the button values
        std::vector<bool> tempButtons;
        std::vector<bool> dpadButtons;


        // fill the array with a false value for each button
        for (int j = 0; j < SDL_JoystickNumButtons(joy); j++)
        {
          tempButtons.push_back(false);
        }

        for (int j = 0; j < NUM_SDL_GPBUTTONS; j++)
        { 
          dpadButtons.push_back(false);
        }

    

        // push the button array into the button state array yo
        m_buttonStates.push_back(tempButtons);
        m_dpadStates.push_back(dpadButtons);

      }
      else
      {
        // if there were ny errors init the controller, than we want to definitely know about it
        std::cout << SDL_GetError();
      }
    }

    // enable the controller events
    SDL_JoystickEventState(SDL_ENABLE);

    m_bJoysticksInitialized = true;

    std::cout << "Initialized " << m_joysticks.size() << " " << "Joystick(s)" << std::endl;
  }
  else
  {
    m_bJoysticksInitialized = false;
  }

}

void InputManager::update()
{
  if (TheInputManager::Instance()->joysticksInitialized() && TheInputManager::Instance()->getJoystickSize() > 1)
  {
    m_mousePosition->x += (float)(TheInputManager::Instance()->getAxisX(1, 2) * 14);
    m_mousePosition->y += (float)(TheInputManager::Instance()->getAxisY(1, 2) * 14);
  }

  SDL_Event event;

  while (SDL_PollEvent(&event))
  {
    switch (event.type)
    {
      case SDL_JOYDEVICEADDED:   initializeJoysticks();       break;
      case SDL_JOYDEVICEREMOVED: removeJoystick(event);       break;
      case SDL_MOUSEMOTION:      onMouseMove(event);          break;
      case SDL_MOUSEBUTTONDOWN:  onMouseButtonDown(event);    break;
      case SDL_MOUSEBUTTONUP:    onMouseButtonUp(event);      break;
      case SDL_KEYDOWN:          onKeyDown(event);            break;
      case SDL_KEYUP:            onKeyUp(event);              break;
      case SDL_JOYAXISMOTION:    onJoystickAxisMove(event);   break;
      case SDL_JOYBUTTONDOWN:    onJoystickButtonDown(event); break;
      case SDL_JOYBUTTONUP:      onJoystickButtonUp(event);   break;
      case SDL_WINDOWEVENT:      Context::OnWindowEvent(event);
                                 OnWindowEvent(event);        break;
      case SDL_JOYHATMOTION: 
      if (event.jhat.value & SDL_HAT_UP)
      {
        onDpadDown(event);
      }
      else if (event.jhat.value & SDL_HAT_DOWN)
      {
        onDpadDown(event);
      }
      else if (event.jhat.value & SDL_HAT_RIGHT)
      {
        onDpadDown(event);
      }
      else if (event.jhat.value & SDL_HAT_LEFT)
      {
        onDpadDown(event);
      }
      else
      {
        onDpadUp(event);
      }
      break;
    }
  }
}

void InputManager::clean()
{
  // close the joysticks brah
  if (m_bJoysticksInitialized)
  {
    for (int i = 0; i < SDL_NumJoysticks(); i++)
    {
      SDL_JoystickClose(m_joysticks[i]);
    }
  }
}

void InputManager::reset()
{
  _mouseButtonDown[LEFT] = false;
  _mouseButtonDown[RIGHT] = false;
  _mouseButtonDown[MIDDLE] = false;
}



// code for keyboard events
bool InputManager::isKeyDown(SDL_Scancode key)
{
  if (_down[key])
  {
    return true;
  }
  return false;
}

bool InputManager::IsKeyPressed(SDL_Scancode key)
{
  if (_down[key] && !_pressed[key])
  {
    _pressed[key] = true;
    return true;
  }
  return false;
}

bool InputManager::getButtonState(int joy, int buttonNumber)
{
  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {
      if (InputManager::Instance()->joysticksInitialized())
      {
        if (m_buttonStates[joy][buttonNumber] && !_gppressed[buttonNumber])
        {
          _gppressed[buttonNumber] = true;
          return true;
        }

      }

      return false;
    }
  }
  else
  {
    if (InputManager::Instance()->joysticksInitialized())
    {
      if (m_buttonStates[joy][buttonNumber] && !_gppressed[buttonNumber])
      {
        _gppressed[buttonNumber] = true;
        return true;
  }

    }

  return false;
}

  return false;
}

bool InputManager::isMousePressed(mouse_buttons button)
{
  if (_mouseButtonDown[button] && !_mouseButtonPressed[button]) {
    _mouseButtonPressed[button] = true;
    return true;
  }
  return false;
}

bool InputManager::getDpadState(int joy, int type)
{

  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {
      if (InputManager::Instance()->joysticksInitialized())
      {
        if (m_dpadStates[joy][type] && !dpadPressed[type])
        {
          dpadPressed[type] = true;
          return true;
        }
        return false;
      }

      return false;
    }
  }
  else
  {
    if (InputManager::Instance()->joysticksInitialized())
    {
      if (m_dpadStates[joy][type] && !dpadPressed[type])
      {
        dpadPressed[type] = true;
        return true;
      }
      return false;
    }

    return false;
  }
  
  return false;
}


// code for mouse events
bool InputManager::getMouseButtonState(int buttonNumber)
{
  return _mouseButtonDown[buttonNumber];
}

glm::vec2* InputManager::getMousePosition() const
{
  return m_mousePosition;
}

int InputManager::getAxisXPressed(int joy, int stick)
{
  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {
      _xAxisWasPressed = false;
      _xAxisIsPressed = getAxisX(joy, stick);

      if (getAxisX(joy, stick) > 0)
      {
        if (_xAxisIsPressed && !_xAxisWasPressed)
          return 1;
      }
      else
      {
        if (_xAxisIsPressed && !_xAxisWasPressed)
          return 2;
      }

      _xAxisWasPressed = _xAxisIsPressed;

      return 0;
    }
  }
  else
  {
    _xAxisWasPressed = false;
    _xAxisIsPressed = getAxisX(joy, stick);

    if (getAxisX(joy, stick) > 0)
    {
      if (_xAxisIsPressed && !_xAxisWasPressed)
        return 1;
    }
    else
    {
      if (_xAxisIsPressed && !_xAxisWasPressed)
        return 2;
    }

    _xAxisWasPressed = _xAxisIsPressed;

    return 0;
  }
  

  return 0;
}

int InputManager::getAxisYPressed(int joy, int stick)
{
  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {

      _yAxisIsPressed = getAxisY(joy, stick);

      //std::cout << getAxisYPressed(joy, stick) << std::endl;

      if (getAxisY(joy, stick) > 0) // DOWN
      {
        if (_yAxisIsPressed && !_yAxisWasPressed)
        {
          _yAxisWasPressed = true;
          return 2;
        }
      }
      else // UP
      {
        if (_yAxisIsPressed && !_yAxisWasPressed)
        {
          _yAxisWasPressed = true;
          return 1;
        }
      }

      _yAxisWasPressed = _yAxisIsPressed;

      return 0;
    }
  }
  else
  {
    _yAxisIsPressed = getAxisY(joy, stick);

    //std::cout << getAxisYPressed(joy, stick) << std::endl;

    if (getAxisY(joy, stick) > 0) // DOWN
    {
      if (_yAxisIsPressed && !_yAxisWasPressed)
      {
        _yAxisWasPressed = true;
        return 2;
      }
    }
    else // UP
    {
      if (_yAxisIsPressed && !_yAxisWasPressed)
      {
        _yAxisWasPressed = true;
        return 1;
      }
    }

    _yAxisWasPressed = _yAxisIsPressed;

    return 0;
  }

  return 0;
}

// code for controller events
float InputManager::getAxisX(int joy, int stick)
{
  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {
      
      if (stick == 1)
      {
        return m_joystickValues[joy].first->x;
      }
      else if (stick == 2)
      {
        return m_joystickValues[joy].second->x;
      }

      return 0;
    }
  }
  else
  {
  if (m_joystickValues.size() > 0)
  {
    if (stick == 1)
    {
      return m_joystickValues[joy].first->x;
    }
    else if (stick == 2)
    {
      return m_joystickValues[joy].second->x;
    }
  }
  return 0;
}

  return 0;
}

int InputManager::getJoystickSize()
{
  return m_joystickValues.size();
}

float InputManager::getAxisY(int joy, int stick)
{
  if (joy > 0)
  {
    if (m_joystickValues.size() > 1)
    {

      if (stick == 1)
      {
        return m_joystickValues[joy].first->y;
      }
      else if (stick == 2)
      {
        return m_joystickValues[joy].second->y;
      }

      return 0;
    }
  }
  else
  {
  if (m_joystickValues.size() > 0)
  {
    if (stick == 1)
    {
      return m_joystickValues[joy].first->y;
    }
    else if (stick == 2)
    {
      return m_joystickValues[joy].second->y;
    }
  }
  return 0;
}

  return 0;
}

float InputManager::getRightTrigger(int joy)
{
  if (m_joystickValues.size() > 0)
  {
    if (m_triggerValues[joy].first->y == 1)
    {
      return 1;
}
  }
  return 0;
}

float InputManager::getLeftTrigger(int joy)
{
  if (m_joystickValues.size() > 0)
  {
    if (m_triggerValues[joy].second->y == 1)
    {
      return 1;
    }
  }
  return 0;
}

// operator function
//InputManager& InputManager::operator=(const InputManager&)
//{

//}


// logic for handling mouse events
void InputManager::onMouseMove(SDL_Event& event)
{
  m_mousePosition->x = (float)event.motion.x;
  m_mousePosition->y = (float)event.motion.y;

}

void InputManager::onMouseButtonDown(SDL_Event& event)
{
  // left click
  if (event.button.button == SDL_BUTTON_LEFT)
    _mouseButtonDown[LEFT] = true;

  // middle click
  if (event.button.button == SDL_BUTTON_MIDDLE)
    _mouseButtonDown[MIDDLE] = true;

  // right click
  if (event.button.button == SDL_BUTTON_RIGHT)
    _mouseButtonDown[RIGHT] = true;
}

void InputManager::onMouseButtonUp(SDL_Event& event)
{
  // left click release
  if (event.button.button == SDL_BUTTON_LEFT) {
    _mouseButtonDown[LEFT] = false;
    _mouseButtonPressed[LEFT] = false;
  }
  // middle click release
  if (event.button.button == SDL_BUTTON_MIDDLE) {
    _mouseButtonDown[MIDDLE] = false;
    _mouseButtonPressed[LEFT] = false;
  }
  // right click release
  if (event.button.button == SDL_BUTTON_RIGHT) {
    _mouseButtonDown[RIGHT] = false;
    _mouseButtonPressed[LEFT] = false;
  }
}

// logic for handling keyboard events
void InputManager::onKeyDown(SDL_Event & event)
{
  _down[event.key.keysym.scancode] = true;
}

void InputManager::onKeyUp(SDL_Event & event)
{
  _down[event.key.keysym.scancode] = false;
  _pressed[event.key.keysym.scancode] = false;
}

void InputManager::onDpadDown(SDL_Event& event)
{
  int whichOne = event.jaxis.which;
  int value = event.jhat.value;

 m_dpadStates[whichOne][value] = true;

 
}

void InputManager::onDpadUp(SDL_Event& event)
{
  int whichOne = event.jaxis.which;
  for (int i = 0; i < NUM_SDL_GPBUTTONS; ++i)
  {
    m_dpadStates[whichOne][i] = false;

    dpadPressed[i] = false;
}
}

/*
void InputManager::onAxisUp(SDL_Event& event)
{
  int whichOne = event.jaxis.which;
  for (int i = 0; i < NUM_SDL_GPBUTTONS; ++i)
  {
    m_dpadStates[whichOne][i] = false;

    dpadPressed[i] = false;
  }
}

void InputManager::onAxisDown(SDL_Event& event)
{
  int whichOne = event.jaxis.which;
  int value = event.j

   m_axisStates[whichOne][value] = true;
}
*/

void InputManager::onJoystickButtonDown(SDL_Event& event)
{

  int whichOne = event.jaxis.which;

  m_buttonStates[whichOne][event.jbutton.button] = true;
}

void InputManager::onJoystickButtonUp(SDL_Event& event)
{
  // controller stick release


  int whichOne = event.jaxis.which;

  m_buttonStates[whichOne][event.jbutton.button] = false;
  _gppressed[event.jbutton.button] = false;
}


// logic for handling controller events
void InputManager::onJoystickAxisMove(SDL_Event& event)
{
  //int axis = event.jaxis.axis;
  //std::cout << axis << std::endl;

  int whichOne = event.jaxis.which;


  // left stick move left or right
  if (event.jaxis.axis == 0)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_joystickValues[whichOne].first->x = 1; 
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_joystickValues[whichOne].first->x = -1;
    }
    else
    {
      m_joystickValues[whichOne].first->x = 0;
    }
  }

  // left stick move up or down
  if (event.jaxis.axis == 1)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_joystickValues[whichOne].first->y = 1;
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_joystickValues[whichOne].first->y = -1;
    }
    else
    {
      m_joystickValues[whichOne].first->y = 0;
    }
  }

  // right stick move left or right
  if (event.jaxis.axis == 3)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_joystickValues[whichOne].second->x = 1;
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_joystickValues[whichOne].second->x = -1;
    }
    else
    {
      m_joystickValues[whichOne].second->x = 0;
    }
  }

  // right stick move up or down
  if (event.jaxis.axis == 4)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_joystickValues[whichOne].second->y = 1;
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_joystickValues[whichOne].second->y = -1;
    }
    else
    {
      m_joystickValues[whichOne].second->y = 0;
    }
  }
  
  // right trigger
  if (event.jaxis.axis == 5)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_triggerValues[whichOne].first->y = 1;
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_triggerValues[whichOne].first->y = -1;
    }
    else
    {
      m_triggerValues[whichOne].first->y = 0;
    }
  }

  // left trigger
  if (event.jaxis.axis == 2)
  {
    if (event.jaxis.value > m_joystickDeadZone)
    {
      m_triggerValues[whichOne].second->y = 1;
    }
    else if (event.jaxis.value < -m_joystickDeadZone)
    {
      m_triggerValues[whichOne].second->y = -1;
    }
    else
    {
      m_triggerValues[whichOne].second->x = 0;
    }
  }


}


/*****************************************************************************/
/*!
\brief
  When SDL encounters a window event, this function is called to find out what
  kind of window event was sent. It will then perform whatever processes need
  to be completed on that event
  https://wiki.libsdl.org/SDL_WindowEvent

\param event
  The event containing the SDL window event.
*/
/*****************************************************************************/

int tabbed = 0;
void InputManager::OnWindowEvent(SDL_Event & event)
{
  switch (event.window.event)
  {
    case SDL_WINDOWEVENT_CLOSE:
      Context::Close();
      break;
    case SDL_WINDOWEVENT_FOCUS_LOST: 
      audEngine.PauseAllChannels();
      // we don't need to do anything if the
      // pause menu is live.
      if (!StateFactory::Active(MENU_PAUSE))
        Global::SpaceManager::PauseAll();
      tabbed += 1;
      break;
    case SDL_WINDOWEVENT_FOCUS_GAINED:
      if(tabbed >= 1)
        audEngine.PauseAllChannels();
      // we don't need to do anything if the pause menu is live.
      if(!StateFactory::Active(MENU_PAUSE))
        Global::SpaceManager::ResumeAll();
      break;
  }
}