/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#ifndef INPUT_H
#define INPUT_H

#include <SDL.h>
#include <vector>
#include <glm/glm.hpp>

#define NUM_SDL_SCANCODES 285
#define NUM_SDL_GPBUTTONS 60


enum mouse_buttons
{
  LEFT,    // 0
  MIDDLE,  // 1
  RIGHT,   // 2
  NUMMOUSEBUTTONS
};


class InputManager
{
public:

	static InputManager* Instance()
	{
		if (s_pInstance == 0)
		{
			s_pInstance = new InputManager();
		}

		return s_pInstance;
	}

	void update();
	void clean();
	void reset();


	// code for keyboard events
	bool isKeyDown(SDL_Scancode key);
  bool IsKeyPressed(SDL_Scancode key);

	// code for mouse events
	bool getMouseButtonState(int buttonNumber);
  bool isMousePressed(mouse_buttons button);
	glm::vec2* getMousePosition() const;

	// code for controller events
	float getAxisX(int joy, int stick);
	float getAxisY(int joy, int stick);
  float getAxisZ(int joy, int stick);
  float getRightTrigger(int joy);
  float getLeftTrigger(int joy);
  int getJoystickSize();
  int getAxisXPressed(int joy, int stick);
  int getAxisYPressed(int joy, int stick);
	bool getButtonState(int joy, int buttonNumber);
  bool getDpadState(int joy, int type);
  bool _hatUp;
  bool _hatDown;
  bool _hatRight;
  bool _hatLeft;


	// controller init code 
	void initializeJoysticks();
	bool joysticksInitialized();
 

private:

	// constructors and destructor
	InputManager();
	//InputManager(const InputManager&);
	~InputManager();

	// operator function
	//InputManager& operator=(const InputManager&);


	// make her a single-ton.......
	static InputManager* s_pInstance;


	// logic for handling mouse events
	void onMouseMove(SDL_Event& event);
	void onMouseButtonDown(SDL_Event& event);
	void onMouseButtonUp(SDL_Event& event);

	// logic for handling keyboard events
	void onKeyDown(SDL_Event & event);
	void onKeyUp(SDL_Event & event);


	// logic for handling controller events
	void onJoystickAxisMove(SDL_Event& event);
	void onJoystickButtonDown(SDL_Event& event);
	void onJoystickButtonUp(SDL_Event& event);
  void onDpadDown(SDL_Event& event);
  void onDpadUp(SDL_Event& event);
  void onAxisUp(SDL_Event& event);
  void onAxisDown(SDL_Event& event);
  void removeJoystick(SDL_Event& event);

  // Used for window events (resize, close, etc.)
  void OnWindowEvent(SDL_Event & event);


	// controller vars
	std::vector<SDL_Joystick*> m_joysticks;
	std::vector<std::pair<glm::vec2*, glm::vec2*>> m_joystickValues;
  std::vector<std::pair<glm::vec2*, glm::vec2*>> m_triggerValues;
	std::vector<std::vector<bool>> m_buttonStates;
  std::vector<std::vector<bool>> m_dpadStates;


	bool m_bJoysticksInitialized;
	static const int m_joystickDeadZone = 10000;


	// keyboard vars
	bool _down[NUM_SDL_SCANCODES];
  bool _pressed[NUM_SDL_SCANCODES];
  bool _gpdown[NUM_SDL_GPBUTTONS];
  bool _gppressed[NUM_SDL_GPBUTTONS];
  bool dpadDown[NUM_SDL_GPBUTTONS];
  bool dpadPressed[NUM_SDL_GPBUTTONS];
  bool axisPressed[4][4];
  bool _yAxisIsPressed;
  bool _yAxisWasPressed;
  bool _xAxisIsPressed;
  bool _xAxisWasPressed;
  bool keyPressed;

  // mouse vars
  bool _mouseButtonDown[NUMMOUSEBUTTONS];
  bool _mouseButtonPressed[NUMMOUSEBUTTONS];
  glm::vec2* m_mousePosition;
};

typedef InputManager TheInputManager;

#endif
