/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file JsonLevelReader.cpp
\author Zach Dolph
\par E-mail: z.dolph\@digipen.edu
\par Project: Boat Game
\date 2/7/2017
\brief
Implementation file for the Json level reader which reads and level files and
creates objects
*/
/*****************************************************************************/

#include "JsonLevelReader.h"
#include "Component.h"
#include "GameObjectManager.h"
#include "SpritePool.h"
#include "ShaderPool.h"
#include "TexturePool.h"

bool JsonLevelReader::CheckDependencies(Json::Value object)
{
  for each (std::string component in object.getMemberNames())
  {
    if (component != "Name" && component != "Sprite" && component != "Transform")
    {
      for each (std::string dependency in Component::GetDependencies(component))
      {
        bool satisfied = false;
        for each (std::string component in object.getMemberNames())
        {
          if (dependency == component)
            satisfied = true;
        }
        if (!satisfied)
          throw DependencyError(dependency);
      }
    }
  }
  return true;
}

bool JsonLevelReader::ValidateComponents(Json::Value object)
{
  for each (std::string component in object.getMemberNames())
  {
    // In short, this line of code gets the component creator for the name of this component
    // then passes in the json value at "Archetype Name" and "Component Name" which contains
    // the data used to initialize the component.
    // This component is not being used, it is just a way to check that the component has the proper data.

    if (component != "Name" && component != "Sprite" && component != "Transform")
    {
      Component * newComponent = Component::GetComponentCreator(component)(object[component]);
      delete newComponent;
    }
  }

  return true;
}

Object * JsonLevelReader::BuildObject(Json::Value object, std::string name, GameObjectManager * objManager)
{
  Object * newObject = objManager->CreateObject(name);
  for each (std::string component in object.getMemberNames())
  {
    // In short, this line of code gets the component creator for the name of this component
    // then passes in the json value at "Archetype Name" and "Component Name" which contains
    // the data used to initialize the component.
    if (component == "Sprite")
    {
      newObject->SetSprite(reinterpret_cast<Sprite *>(SpritePool::AddTexture(&ShaderPool::Texture(), TexturePool::Add(object[component]["texture"].asString()))));
    }
    else if (component == "Transform")
    {
      glm::vec3 Scale = glm::vec3(object[component]["scaleX"].asFloat(), object[component]["scaleY"].asFloat(), 0);
      newObject->GetTransform()->SetScale(Scale);
    }
    else if (component != "Name")
    {
      Component * newComponent = Component::GetComponentCreator(component)(object[component]);
      newObject->Add(newComponent);
    }
  }

  newObject->Init();
  return newObject;
}