/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#include "LevelOne.h"

#include "RendererWater.h"
#include <fstream>
#include "TileMap.h"
#include "JsonLevelReader.h"
#include "InputManager.h"
#include "Space.h"
#include "SpaceManager.h"
#include "Context.h"

void LevelOne::Allocate()
{
  RendererWater::SetPeakColor(glm::vec3(0, 0.5, 1));
  RendererWater::SetTroughColor(glm::vec3(0, 0.3, 0.5));
  std::ifstream inputFile;
  inputFile.open("Resources/levels/Escape.lvl");
  Json::Reader reader;
  Json::Value value;
  bool parsingSuccessful = reader.parse(inputFile, value);
  // Check to make sure the file was read correctly.
  if (!parsingSuccessful)
  {
    std::cout << "parsing failed";
    std::cout << reader.getFormattedErrorMessages();
  }
  else
  {

    TileData data;
    int pixel_scale = value["tilewidth"].asInt();
    float scale = value["properties"]["TileScale"].asFloat();
    data.Scale = scale;
    data.columns = value["tilesets"][0]["columns"].asUInt();
    data.tile_count = value["tilesets"][0]["tilecount"].asUInt();
    data.rows = data.tile_count / data.columns;
    if (data.tile_count % data.columns != 0)
      data.rows++;
    data.spritesheet = value["tilesets"][0]["image"].asString();

    for each(Json::Value layer in value["layers"])
    {
      if (layer["type"].asString() == "tilelayer")
      {
        if (layer["name"].asString() == "Tilemap")
          data.collidable = true;
        else
          data.collidable = false;
        
        _tileMaps.push_back(new TileMap(layer, GetObjectManager(), data));
        continue;
      }
      if (layer["name"].asString() != "Archetypes")
        continue;
      for each(Json::Value object in layer["objects"])
      {
        std::string name = object["type"].asString();
        glm::vec3 pos;
        pos = glm::vec3(object["x"].asFloat(), -object["y"].asFloat(), 0);
        pos /= pixel_scale;
        pos *= scale;

        std::cout << "Position: " << pos.x << ", " << pos.y << std::endl;
        
        _objectManager.CreateArchetypeObjectAtPosition(name, pos);
      }
      for each(Json::Value object in value["Objects"])
      {
        JsonLevelReader::CheckDependencies(object);
        JsonLevelReader::ValidateComponents(object);
        JsonLevelReader::BuildObject(object, object["Name"].asString(), &_objectManager);
      }
    }
    for each(TileMap * tileMap in _tileMaps)
    {
      (**_objectManager.Space()).GetRenderer().Add(tileMap);
    }
  }
}

void LevelOne::Load()
{
}

bool LevelOne::Update()
{

  if (InputManager::Instance()->joysticksInitialized())
    gamepadUpdate();

  // SYSTEMS UPDATE FUNCTIONS
  ///////////////////////////////////
  // 

  for each(TileMap * tilemap in _tileMaps)
  {
    tilemap->Update();
  }

  glm::vec2 mousePos = *InputManager::Instance()->getMousePosition();
  (**_space).ScreenToWorld(mousePos);

  ///////////////////////////////////
  // INPUT CODE FOR LEVEL
  ///////////////////////////////////
  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_ESCAPE))
  {
    Global::SpaceManager::PauseAll();
    Global::SpaceManager::Add(MENU_PAUSE, 30);
  }

  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_I))
  {
    Global::SpaceManager::RemoveAll();
    Global::SpaceManager::Add(WIN_SCREEN, 30);
  }

  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_L))
  {
    Global::SpaceManager::RemoveAll();
    Global::SpaceManager::Add(LOSE_SCREEN, 30);
  }

  if (!StateFactory::Active(GUNNER_INVENTORY) &&
      TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_RSHIFT))
  {
    //Global::SpaceManager::Add(GUNNER_INVENTORY, 30);
  }
  // END OF INPUT CODE FOR LEVEL
  ///////////////////////////////////

  // Adjust water to match world motion
  float cam_width = (**_space).GetCamera()->GetHeight() * Context::AspectRatio();
  float cam_height = (**_space).GetCamera()->GetHeight();
  const glm::vec3 & cam_pos = (**_space).GetCamera()->GetPosition();
  RendererWater::SetWorldSize(glm::vec2(cam_width, cam_height));
  RendererWater::SetCameraPosition(glm::vec2(cam_pos.x, cam_pos.y));

  return true;
}

void LevelOne::Free()
{
  //audEngine.StopEvent(audEngine.GetMusicEvent("LevelOne"));
  _objectManager.ClearObjects();
  for each(TileMap * tileMap in _tileMaps)
  {
    (**_objectManager.Space()).GetRenderer().Remove(tileMap);
    delete tileMap;
  }
}

void LevelOne::gamepadUpdate()
{
  // IF ESCAPE IS PRESSED, Pause
  if (TheInputManager::Instance()->getButtonState(0, 7) || TheInputManager::Instance()->getButtonState(1, 7))
    
  {
    Global::SpaceManager::PauseAll();
    Global::SpaceManager::Add(MENU_PAUSE, 30);
  }
}
