/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file MenuSystem.cpp
\author yourname
\par E-mail: youremail
\par Project: Boat Game
\date 2/24/2017 3:11:05 PM
\brief

\par
Functions Include:
-
*/
/*****************************************************************************/

#include "MenuSystem.h"
#include "Object.h"
#include "GameObjectManager.h"
#include "Transform.h"
#include "InputManager.h"
#include "SoundEmitter.h"
#include "States.h"
#include "Space.h"
#include "SpaceManager.h"


MenuSystem::MenuSystem(glm::vec2 basePos, glm::vec2 offset) : 
  Component("MenuSystem"),
  _cursorBasePos(basePos),
  _cursorOffset(offset),
  _confirmation(false),
  _buttonPosition(0),
  _transition(false),
  _firstFrame(true)
{
}

void MenuSystem::Init()
{
  _cursor = Owner().ObjectManager()->CreateArchetypeObjectAtPosition("menuArrow", glm::vec3(_cursorBasePos.x, _cursorBasePos.y, -1.0f));
  //Owner().ObjectManager()->CreateArchetypeObject("mainMenu");

}

void MenuSystem::Update()
{
  glm::vec3 cursor_position = _cursor->GetTransform()->GetTranslation();

  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_ESCAPE) && !_confirmation)
  {
    for (int i = 0; i < _menuOptions.size(); i++)
    {
      if (_menuOptions[i] == "ResumeGame")
      {
        ResumeGame();
        return;
      }
    }
    _confirmFunction = &MenuSystem::Quit;
    ConfirmQuit();
  }


  int mouse = MousePosition();
  if (!_transition)
  {
    if (mouse != -1)
    {
      if (mouse != _buttonPosition)
      {
        _buttonPosition = mouse;
        Owner().Find<SoundEmitter>()->PlaySound("MenuHover");
      }
    }

    int dir = MenuDirection();
    _buttonPosition += dir;
    if(dir != 0)
      Owner().Find<SoundEmitter>()->PlaySound("MenuHover");
  }

  if (!_confirmation)
  {

    if (_buttonPosition >= (signed)_menuOptions.size())
      _buttonPosition = 0;
    if (_buttonPosition < 0)
      _buttonPosition = _menuOptions.size() - 1;

    _cursor->GetTransform()->Translation() = glm::vec3(_cursorBasePos + _cursorOffset * (float)_buttonPosition, 0);

    //Skip first frame
    if (_firstFrame)
    {
      _firstFrame = false;
      return;
    }
  }
  else
  {
    if (_buttonPosition > 1)
      _buttonPosition = 0;
    if (_buttonPosition < 0)
      _buttonPosition = 1;
    _cursor->GetTransform()->Translation() = glm::vec3(2.5, 1 - _buttonPosition * 3, 0);
  }

  // IF PRESS ENTER
  if ((TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_RETURN) 
    || TheInputManager::Instance()->getButtonState(0, SDL_CONTROLLER_BUTTON_A)
    || (TheInputManager::Instance()->isMousePressed(mouse_buttons::LEFT) && mouse != -1)) && !_transition)
  {
    MenuButtonSelected();
  }

}

/**** INPUT "MACROS" ****/
int MenuSystem::MenuDirection()
{
  int joyFlick = TheInputManager::Instance()->getAxisYPressed(0, 1);
  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_UP)
    || (joyFlick == 1)
    || TheInputManager::Instance()->getDpadState(0, 1))
  {
    return -1;
  }
  else if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_DOWN)
    || (joyFlick == 2)
    || TheInputManager::Instance()->getDpadState(0, 4))
  {
    return 1;
  }
  return 0;
}

int MenuSystem::MousePosition()
{
  glm::vec2 mousePosition = *InputManager::Instance()->getMousePosition();
    (**Owner().ObjectManager()->Space()).ScreenToWorld(mousePosition);
    if (!_confirmation)
    {
      if (mousePosition.x < _cursorBasePos.x - 1.1f && mousePosition.x > _cursorBasePos.x - 18
        && mousePosition.y < _cursorBasePos.y - _cursorOffset.y / 2 && mousePosition.y > _cursorBasePos.y + _cursorOffset.y * (_menuOptions.size() - 0.5f))
        return (int)((mousePosition.y - _cursorBasePos.y + _cursorOffset.y / 2) / _cursorOffset.y);
    }
    else
    {
      glm::vec3 confirmPosition = _confirmMessage->GetTransform()->Translation();
      if (mousePosition.x < confirmPosition.x + 2.0f && mousePosition.x > confirmPosition.x - 2.0f
        && mousePosition.y < confirmPosition.y - _cursorOffset.y - 0.5f && mousePosition.y > confirmPosition.y + _cursorOffset.y - 0.5f)
        return (int)((mousePosition.y - confirmPosition.y + _cursorOffset.y + 0.5f) / _cursorOffset.y);
    }

  return -1;
}

void MenuSystem::StartGame()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  TransitionInfo transition_info(3.0f, 1.0f, LEVEL_ONE, "Resources/levels/Level1.lvl");
  Global::SpaceManager::Add(transition_info);
  _transition = true;
}

void MenuSystem::ControlsScreen()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  (**Owner().ObjectManager()->Space()).Pause();
  (**Owner().ObjectManager()->Space()).GetRenderer().Deactivate();
  Global::SpaceManager::Add(CONTROLS_SCREEN, 30);
}

void MenuSystem::SettingsMenu()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  (**Owner().ObjectManager()->Space()).Pause();
  (**Owner().ObjectManager()->Space()).GetRenderer().Deactivate();
  Global::SpaceManager::Add(SETTINGS_MENU, 30);
}

void MenuSystem::CreditsScreen()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  TransitionInfo transition_info(0.75f, 1.0f, CREDITS_SCREEN);
  Global::SpaceManager::Add(transition_info);
}

void MenuSystem::Quit()
{
  (**Owner().ObjectManager()->Space()).Quit();
}


void MenuSystem::MenuButtonSelected()
{
  if (!_confirmation)
  {
    std::string & option = _menuOptions[_buttonPosition];
    if (option == "StartGame")
      StartGame();
    if (option == "Controls")
      ControlsScreen();
    if (option == "Settings")
      SettingsMenu();
    if (option == "Credits")
      CreditsScreen();
    if (option == "Quit")
    {
      _confirmFunction = &MenuSystem::Quit;
      ConfirmQuit();
    }
    if (option == "MainMenu")
    {
      _confirmFunction = &MenuSystem::MainMenu;
      ConfirmQuit();
    }
    if (option == "CreditsQuit")
    {
      _confirmFunction = &MenuSystem::CreditsScreen;
      ConfirmQuit();
    }
    if (option == "Restart")
      Restart();
    if (option == "ResumeGame")
      ResumeGame();
  }
  else
  {
    switch (_buttonPosition)
    {
    case 0: // YES
            // This is how you call pointers to member functions
      (this->*(_confirmFunction))();
      break;
    case 1: // NO
      _confirmation = false;
      _buttonPosition = _menuOptions.size() - 1;
      Owner().ObjectManager()->DeleteObject(_confirmMessage);
      break;
    default:
      break;
    }
  }
}

void MenuSystem::ConfirmQuit()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  _confirmation = true;
  _buttonPosition = 0;
  _confirmMessage = Owner().ObjectManager()->CreateArchetypeObject("confirmation");
  Owner().ObjectManager()->DeleteObject(_cursor);
  _cursor = Owner().ObjectManager()->CreateArchetypeObject("menuArrow");
}

void MenuSystem::MainMenu()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  TransitionInfo transition_info(2.0f, 1.5f, MENU_MAIN);
  Global::SpaceManager::Add(transition_info);
  _transition = true;
}
void MenuSystem::ResumeGame()
{
  Owner().Find<SoundEmitter>()->PlaySound("MenuStart");
  Global::SpaceManager::Remove(Owner().ObjectManager()->Space());
  Global::SpaceManager::ResumeAll();
}

void MenuSystem::Restart()
{
  Level * level = dynamic_cast<Level *>(Global::SpaceManager::FindState(LEVEL_ONE));
  std::string levelFile = level->GetLevelFile();
  TransitionInfo transition_info(2.0f, 1.5f, LEVEL_ONE, levelFile);
  Global::SpaceManager::Add(transition_info);
  _transition = true;
}

void MenuSystem::AddMenuOption(std::string option)
{
  _menuOptions.push_back(option);
}
Component * CreateMenuSystemComponent(Json::Value value)
{
  MenuSystem * menu = new MenuSystem(glm::vec2(value["CursorPos"][0].asFloat(), value["CursorPos"][1].asFloat()),
    glm::vec2(value["Offset"][0].asFloat(), value["Offset"][1].asFloat()));
  for (int i = 0; i < (signed)value["MenuOptions"].size(); i++)
  {
    menu->AddMenuOption(value["MenuOptions"][i].asString());
  }
  return menu;
}
