/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#pragma once
/*****************************************************************************/
/*!
\file Physics.h
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: Boat Game
\date 10/6/2016
\brief
	The interface for the physics component.
*/
/*****************************************************************************/

#ifndef PHYSICS_H
#define PHYSICS_H

#include "Component.h"
#include "Object.h"
#include <glm/glm.hpp> // vec2

/*****************************************************************************/
/*!
\class Physics
\brief
	Basic Physics component, can be dynamic, kinematic, or static.
	Static component does nothing but is required to use collider on static objects.

\par
	Operations include:
	- Update Physics
	- Apply Forces
	- Set Angular Velocity
	- Get Velocity Value
	- Set Velocity

\deprecated
	-

\bug
	-
*/
/*****************************************************************************/
class Physics : public Component
{
public:

	/*****************************************************************************/
	/*!
	\enum MotionType
	\brief
		Used to set the type of motion that the physics component will use.

	\par
		Members
		-Kinematic
			Uses only velocity to determine position.
		-Dynamic
			Uses forces to determine accel and velocity.
		-Static
			Does not use any physics calculation, but is required to use
			a collider on a static object.
	*/
	/*****************************************************************************/
	enum MotionType
	{
		Kinematic,
		Dynamic,
		Static
	};

	Physics(float mass, MotionType motionType);
	~Physics();
	void Update();
	void ApplyForce(glm::vec2 force);
  void ApplyAngularForce(float force);
	void SetAngularVelocity(float velocity);
  float GetAngularVelocity();
	glm::vec2 GetVelocity();
	void SetVelocity(glm::vec2 velocity);
  float GetMass();
  MotionType GetMotionType();
  void SetDrag(float drag);
  void SetAngularDrag(float drag);
	
private:
  //! Determines how the object is affected by forces
	MotionType _motionType;
  //! Determines the object's resistance to forces
	float _mass;
	//! Totals the sum of forces each frame.
	glm::vec2 _sumForces;
  float _sumAngularForce;
  //! The velocity vector of the object
	glm::vec2 _velocity;
  //! The angular velocity scalar of the object
	float _angularVelocity;
  float _dragPercentage;
  float _angularDrag;
};

// Creator for physics object from json data
Component * CreatePhysicsComponent(Json::Value value);

#endif
