#include "Context.h"
#include "ArchetypeManager.h"
#include "InputManager.h"
#include <SDL.h>
#include "CollisionManager.h"
#include "GameObjectManager.h"
#include "Graphics.h"
#include "GraphicPool.h"
#include "TexturePool.h"
#include "ShaderPool.h"
#include "State.h"

#include "Physics.h"
#include "CircleCollider.h"

#include "dt.h"
#include <iostream>

namespace PhysicsDemo
{

  void Load()
  {
    Context::Create("game");


    shaderID shader = ShaderPool::Add("shaders/test/vertex.sgl", "shaders/test/fragment.sgl");
    graphicID box = GraphicPool::Add("Resources/Mesh/boxt.msh");
    archetypeManager.InitArchetypes();
    Graphics::SetClearColor(0.2f, 0.2f, 0.2f, 1.0f);
    Graphics::BlendMode();
  }

  void Init()
  {
    
  }
  bool Update()
  {
    InputManager::Instance()->update();

    if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_ESCAPE))
    {
      return false;
    }

    static bool space_pressed = false;

    if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_SPACE))
    {
      if (space_pressed == false)
      {
        glm::vec2 posOne = glm::vec2(1.0f, ((float)rand() / (float)RAND_MAX) * 2 - 1);
        glm::vec2 posTwo = glm::vec2(-1.0f, ((float)rand() / (float)RAND_MAX) * 2 - 1);
        Object * one = archetypeManager.CreateArchetypeObjectAtPosition("Physics Object", posOne);
        Object * two = archetypeManager.CreateArchetypeObjectAtPosition("Physics Object", posTwo);
        Physics * onePhys = static_cast<Physics *>(one->Find("Physics"));
        Physics * twoPhys = static_cast<Physics *>(two->Find("Physics"));
        onePhys->SetVelocity((glm::vec2(0, 0) - posOne) * 7.0f);
        twoPhys->SetVelocity((glm::vec2(0, 0) - posTwo) * 7.0f);
        CircleCollider * oneCollider = static_cast<CircleCollider *>(one->Find("CircleCollider"));
        CircleCollider * twoCollider = static_cast<CircleCollider *>(two->Find("CircleCollider"));
        float radOne = (float)rand() / (float)RAND_MAX;
        float radTwo = (float)rand() / (float)RAND_MAX;
      }

      space_pressed = true;
    }
    else
      space_pressed = false;

    CollisionManager::Update();

    objectManager.Update();

    return true;
  }

  void Draw()
  {
    Context::Swap();
    Graphics::Clear();
  }


  void Free()
  {
  }


  void Unload()
  {
  }

  State physicsLevel(Load, Init, Update, Draw, Free, Unload);
}