/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file RendererWater.coo
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 09/04/2017
*/
/*****************************************************************************/

#include <GL/glew.h>

#include "ShaderPool.h"
#include "GraphicPool.h"
#include "Context.h"
#include "Time.h"

#include "RendererWater.h"

/*!
\brief Renders the water to the currently bound framebuffer.
*/
void RendererWater::Render()
{
  ShaderPool::Water().Use();
  glUniform1f(ShaderPool::Water().TimeU(), Time::TotalTime());
  GraphicPool::Screen().Draw();
}

/*!
\brief Set the players positon on the water.
\param The players current world position.
*/
void RendererWater::SetCameraPosition(const glm::vec2 & camera_position)
{
  ShaderPool::Water().Use();
  glUniform2f(ShaderPool::Water().CameraPositionU(),
    camera_position.x, camera_position.y);
}

/*!
\brief Set the wave frequency of the water shader.
\param The water shader's new wave frequency.
*/
void RendererWater::SetFrequency(float new_frequency)
{
  ShaderPool::Water().Use();
  glUniform1f(ShaderPool::Water().FrequencyU(), new_frequency);
}

/*!
\brief Set the speed multiple of the water shader.
\param new_speed_multiple The new speed multiple.
*/
void RendererWater::SetSpeedMultiple(float new_speed_multiple)
{
  ShaderPool::Water().Use();
  glUniform1f(ShaderPool::Water().SpeedMultipleU(), new_speed_multiple);
}

/*!
\brief Set the world size used for the water shader.
\param The world size used for the water shader.
*/
void RendererWater::SetWorldSize(const glm::vec2 & new_world_size)
{
  ShaderPool::Water().Use();
  glUniform2f(ShaderPool::Water().WorldSizeU(), 
    new_world_size.x, new_world_size.y);
}

/*!
\brief Set the number of layers in the water shader.
\param new_layers The number of layers that the water will render with.
*/
void RendererWater::SetLayers(int new_layers)
{
  ShaderPool::Water().Use();
  glUniform1i(ShaderPool::Water().LayersU(), new_layers);
}

/*!
\brief Set the color of wave peaks.
\param new_peak_color The new color of wave peaks.
*/
void RendererWater::SetPeakColor(const glm::vec3 & new_peak_color)
{
  ShaderPool::Water().Use();
  glUniform4f(ShaderPool::Water().PeakColorU(), 
    new_peak_color.x, new_peak_color.y, new_peak_color.z, 1.0f);
}

/*!
\brief Set the color of wave troughs.
\param new_trough_color The new color of wave troughs.
*/
void RendererWater::SetTroughColor(const glm::vec3 & new_trough_color)
{
  ShaderPool::Water().Use();
  glUniform4f(ShaderPool::Water().TroughColorU(),
    new_trough_color.x, new_trough_color.y, new_trough_color.z, 1.0f);
}