/*****************************************************************************/
/*!
\file SandboxLevel.cpp
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: Boat Game
\date 10/07/2016
\brief
  This file implements code that would previously be in main, and puts it 
  in a level to test the game state manager.
*/
/*****************************************************************************/


#include <GL\glew.h>
#include <SDL.h>
#include <SDL_opengl.h>

//#include <ft2build.h>
//#include FT_FREETYPE_H

#include "State.h"
#include "GameObjectManager.h"
#include "Context.h"
#include "ShaderPool.h"
#include "GraphicPool.h"
#include "TexturePool.h"
#include "Graphics.h"
#include "AudioEngine.h"
#include "InputManager.h"
#include "Physics.h"
#include "CircleCollider.h"
#include "ArchetypeManager.h"
#include "Transform.h"
#include "Sprite.h"
#include "CharacterController.h"
#include "GlobalCamera.h"
#include "GlobalTransform.h"
#include "Camera.h"
#include "CollisionManager.h"
#include "GlobalCamera.h"
#include "SimpleForceController.h"

#include "Text.h"

#include "dt.h"

namespace SandboxLevel
{

  //FT_Face courier_new_plain;

  AudioEngine audEngine;
  void Load()
  {
    Context::Create("game");
    // sound code
    audEngine.Init();
    /*
    FT_Error error = FT_New_Face(library, 
      "Resources/fonts/cour.ttf", 0, &courier_new_plain);*/

    //Testing archetype manager code
    archetypeManager.InitArchetypes();
  }

  void Init()
  {
    // sound code
    audEngine.LoadBank("Resources/audio/Master Bank.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    audEngine.LoadBank("Resources/audio/Master Bank.strings.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    audEngine.LoadBank("Resources/audio/Music.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    audEngine.LoadBank("Resources/audio/SFX.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    audEngine.LoadBank("Resources/audio/Dialogue.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    audEngine.LoadBank("Resources/audio/Ambiance.bank", FMOD_STUDIO_LOAD_BANK_NORMAL);
    //audEngine.PlayEvent("event:/Music/Musical Style Sample");


    Graphics::SetClearColor(0.2f, 0.2f, 0.2f, 1.0f);
    Graphics::BlendMode();

    archetypeManager.CreateArchetypeObjectAtPosition("Player", glm::vec2(0, 0));
    archetypeManager.CreateArchetypeObjectAtPosition("Enemy", glm::vec2(1, 0));
    
  }

  bool Update()
  {
    InputManager::Instance()->update();



    if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_ESCAPE))
    {
      return false;
    }



    audEngine.Update();

    CollisionManager::Update();

    objectManager.Update();

    return true;
  }

  // Temporary helper function
  void PrintPerformance()
  {
    // Performance tracking code
    //FT_GlyphSlot slot = courier_new_plain->glyph;
    int pos_x = 0;
    int pos_y = 0;
    int framespersecond = static_cast<int>(1 / dt);
    std::string frametext = "FPS: " + framespersecond;
    size_t length = frametext.length();
    /*
    for (int i = 0; i < length; i++)
    {
      FT_Error error = FT_Load_Char(courier_new_plain,
        frametext[i], FT_LOAD_RENDER);
      DrawChar(&slot->bitmap,
        pos_x + slot->bitmap_left,
        pos_y + slot->bitmap_top);
      //Advance text position
      pos_x += slot->advance.x >> 6;
    }
    */
  }

  void Draw()
  {
    Context::Swap();
    Graphics::Clear();
    //PrintPerformance();
  }


  void Free()
  {
  }

  void Unload()
  {
    audEngine.ShutDown();
  }


  State sandboxLevel(Load, Init, Update, Draw, Free, Unload);
}