/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */

#include <glm/glm.hpp>

#include "SpaceManager.h"
#include "Space.h"
#include "StateFactory.h"
#include "InputManager.h"
#include "GameObjectManager.h"
#include "Transform.h"
#include "Sprite.h"
#include "ShaderPool.h"
#include "SpritePool.h"
#include "AudioEngine.h"
#include "TexturePool.h"
#include "Time.h"
#include "Context.h"

#include "SettingsMenu.h"

#define LOCX           -8.0f
#define TOPY            9.16666667
#define CHANGEY         2.91666667
#define BUTTONHALFWIDHT 1.45833334

#define LEFTBOUNDX     -25.8f
#define RIGHTBOUNDX    -9.5f

#define FULLSCREENTOPY   TOPY - CHANGEY * FULLSCREEN + BUTTONHALFWIDHT
#define MASTERVOLUMETOPY TOPY - CHANGEY * MASTER_VOLUME + BUTTONHALFWIDHT
#define MUSICVOLUMETOPY  TOPY - CHANGEY * MUSIC_VOLUME + BUTTONHALFWIDHT
#define SFXVOLUMETOPY    TOPY - CHANGEY * SFX_VOLUME + BUTTONHALFWIDHT
#define BACKTOPY         TOPY - CHANGEY * BACK + BUTTONHALFWIDHT

#define SLIDERRADIUS 1.55f
#define SLIDERRANGE  28.0f

namespace
{
  Transform * SettingsTran;
  Transform * cursor_tran;
}

SettingsMenu::SettingsMenu() : _currentOption(BACK), _previousOption(BACK)
{}

void SettingsMenu::Allocate()
{
  const Texture * SettingsTexture = TexturePool::Add("Resources/Visual/Menu/settings.png");
  const Texture * CursorTexture = TexturePool::Add("Resources/Visual/Menu/cursor.png");

  _objectManager.CreateArchetypeObject("mouseCursor");

  // menu
  Object * settings = _objectManager.CreateObject("settings");
  SettingsTran = settings->GetTransform();
  settings->SetSprite(reinterpret_cast<Sprite *>(SpritePool::AddTexture(&ShaderPool::Texture(), SettingsTexture)));
  settings->Init();
  // cursor
  Object * menuSelec = _objectManager.CreateObject("menuSelec");
  cursor_tran = menuSelec->GetTransform();
  menuSelec->SetSprite(reinterpret_cast<Sprite *>(SpritePool::AddTexture(&ShaderPool::Texture(), CursorTexture)));
  menuSelec->Init();
  // slider objects
  Object * master_slider;
  master_slider = _objectManager.CreateArchetypeObject("circleSlider");
  _masterVSlider = master_slider->GetTransform();
  Object * music_slider;
  music_slider = _objectManager.CreateArchetypeObject("circleSlider");
  _musicVSlider = music_slider->GetTransform();
  Object * sfx_slider;
  sfx_slider = _objectManager.CreateArchetypeObject("circleSlider");
  _sfxVSlider = sfx_slider->GetTransform();

  // deactivating main menu render
  spaceID menu_space = Global::SpaceManager::SpaceUnderneath(_space);
  (**menu_space).GetRenderer().Deactivate();
}

void SettingsMenu::Load()
{
  SettingsTran->Scale() = glm::vec3(30.0f, 30.0f, 1.0f);
  cursor_tran->Scale() = glm::vec3(3.25f, 3.25f, 1.0f);
  cursor_tran->Translation() = glm::vec3(-8.0f, -2.5f, 0.0f);
  // initialize volume slider translations
  float volume = audEngine.GetVolumeAllChannels();
  _masterVSlider->Translation() = glm::vec3(-6.0f + volume * SLIDERRANGE,
    TOPY - MASTER_VOLUME * CHANGEY, 1.0f);
  volume = audEngine.GetBusGroupVolume("bus:/Music");
  _musicVSlider->Translation() = glm::vec3(-6.0f + volume * SLIDERRANGE,
    TOPY - MUSIC_VOLUME * CHANGEY, 1.0f);
  volume = audEngine.GetBusGroupVolume("bus:/SFX");
  _sfxVSlider->Translation() = glm::vec3(-6.0f + volume * SLIDERRANGE,
    TOPY - SFX_VOLUME * CHANGEY, 1.0f);
}

/*!
\brief Updates according to the current option.
*/
bool SettingsMenu::Update()
{
  _previousOption = _currentOption;
  SetOption();
  if (_previousOption != _currentOption)
    audEngine.PlayEvent("event:/UISFX/MenuHover");
  switch (_currentOption)
  {
  case SettingsMenu::FULLSCREEN:
    FullscreenUpdate();
    break;
  case SettingsMenu::MASTER_VOLUME:
    MasterVolumeUpdate();
    break;
  case SettingsMenu::MUSIC_VOLUME:
    MusicVolumeUpdate();
    break;
  case SettingsMenu::SFX_VOLUME:
    SFXVolumeUpdate();
    break;
  case SettingsMenu::BACK:
    BackUpdate();
    break;
  default:
    break;
  }
  _prevMouseLoc = _mouseLoc;
  return true;
}

/*!
\brief Finds witch option is currently in use.
*/
void SettingsMenu::SetOption()
{
  // keyboard input
  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_UP)) {
    switch (_currentOption) {
    case SettingsMenu::MASTER_VOLUME:
      _currentOption = FULLSCREEN;
      break;
    case SettingsMenu::MUSIC_VOLUME:
      _currentOption = MASTER_VOLUME;
      break;
    case SettingsMenu::SFX_VOLUME:
      _currentOption = MUSIC_VOLUME;
      break;
    case SettingsMenu::BACK:
      _currentOption = SFX_VOLUME;
      break;
    }
  }
  else if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_DOWN)) {
    switch (_currentOption)
    {
    case SettingsMenu::FULLSCREEN:
      _currentOption = MASTER_VOLUME;
      break;
    case SettingsMenu::MASTER_VOLUME:
      _currentOption = MUSIC_VOLUME;
      break;
    case SettingsMenu::MUSIC_VOLUME:
      _currentOption = SFX_VOLUME;
      break;
    case SettingsMenu::SFX_VOLUME:
      _currentOption = BACK;
      break;
    }
  }
  // mouse input
  _mouseLoc = *InputManager::Instance()->getMousePosition();
  (**_space).ScreenToWorld(_mouseLoc);
  if (_mouseLoc.x > LEFTBOUNDX && _mouseLoc.x < RIGHTBOUNDX) {
    if (_mouseLoc.y < FULLSCREENTOPY        && _mouseLoc.y > MASTERVOLUMETOPY)
      _currentOption = FULLSCREEN;
    else if (_mouseLoc.y < MASTERVOLUMETOPY && _mouseLoc.y > MUSICVOLUMETOPY)
      _currentOption = MASTER_VOLUME;
    else if (_mouseLoc.y < MUSICVOLUMETOPY  && _mouseLoc.y > SFXVOLUMETOPY)
      _currentOption = MUSIC_VOLUME;
    else if (_mouseLoc.y < SFXVOLUMETOPY    && _mouseLoc.y > BACKTOPY)
      _currentOption = SFX_VOLUME;
    else if (_mouseLoc.y < BACKTOPY         && _mouseLoc.y > BACKTOPY - CHANGEY)
      _currentOption = BACK;
  }

  // mouse slider edge cases
  // don't ask why I am using the previous
  glm::vec2 master_slider(_masterVSlider->GetTranslation().x, 
      _masterVSlider->GetTranslation().y);
  _masterDistance = glm::distance(_prevMouseLoc, master_slider);
  glm::vec2 music_slider(_musicVSlider->GetTranslation().x, 
    _musicVSlider->GetTranslation().y);
  _musicDistance = glm::distance(_prevMouseLoc, music_slider);
  glm::vec2 sfx_slider(_sfxVSlider->GetTranslation().x,
    _sfxVSlider->GetTranslation().y);
  _sfxDistance = glm::distance(_prevMouseLoc, sfx_slider);
  // on master volume slider
  if (_masterDistance < SLIDERRADIUS)
    _currentOption = MASTER_VOLUME;
  else if (_musicDistance < SLIDERRADIUS)
    _currentOption = MUSIC_VOLUME;
  else if (_sfxDistance < SLIDERRADIUS)
    _currentOption = SFX_VOLUME;
  cursor_tran->Translation() = glm::vec3(LOCX, TOPY - _currentOption * CHANGEY, 
    0.0f);
}

/*!
\brief Update for fullscreen option
*/
void SettingsMenu::FullscreenUpdate()
{
  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_RETURN) ||
    TheInputManager::Instance()->isMousePressed(LEFT)) {
    audEngine.PlayEvent(audEngine.GetMusicEvent("MenuStart"));
    Context::ToggleFullscreen();
  }
}

/*!
\brief Update for master volume option
*/
void SettingsMenu::MasterVolumeUpdate()
{
  // keyboard
  float current_volume = audEngine.GetVolumeAllChannels();
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_LEFT))
    current_volume -= 1.0f * Time::DT();
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_RIGHT))
    current_volume += 1.0f * Time::DT();
  // mouse
  if (InputManager::Instance()->getMouseButtonState(LEFT) && 
    _masterDistance < SLIDERRADIUS)
    current_volume = current_volume + (_mouseLoc.x - _prevMouseLoc.x) / 28.0f;
  // set
  current_volume = Clamp(0.0f, 1.0f, current_volume);
  audEngine.SetVolumeAllChannels(current_volume);
  _masterVSlider->Translation() = glm::vec3(-6.0f + current_volume * SLIDERRANGE,
    TOPY - CHANGEY * MASTER_VOLUME, 1);

}

/*!
\brief Update for when Music volume option is selected.
*/
void SettingsMenu::MusicVolumeUpdate()
{
  // keyboard
  float current_volume = audEngine.GetBusGroupVolume("bus:/Music");
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_LEFT))
    current_volume -= 1.0f * Time::DT();
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_RIGHT))
    current_volume += 1.0f * Time::DT();
  // mouse 
  if (InputManager::Instance()->getMouseButtonState(LEFT) &&
    _musicDistance < SLIDERRADIUS)
    current_volume = current_volume + (_mouseLoc.x - _prevMouseLoc.x) / 28.0f;
  // set
  current_volume = Clamp(0.0f, 1.0f, current_volume);
  audEngine.SetBusGroupVolume("bus:/Music", current_volume);
  _musicVSlider->Translation() = glm::vec3(-6.0f + current_volume * 28,
    TOPY - CHANGEY * MUSIC_VOLUME, 1);
}

/*!
\brief Update for when SFX volume option is selected.
*/
void SettingsMenu::SFXVolumeUpdate()
{
  // keybarod
  float current_volume = audEngine.GetBusGroupVolume("bus:/SFX");
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_LEFT))
    current_volume -= 1.0f * Time::DT();
  if (TheInputManager::Instance()->isKeyDown(SDL_SCANCODE_RIGHT))
    current_volume += 1.0f * Time::DT();
  // mouse
  if (InputManager::Instance()->getMouseButtonState(LEFT) &&
    _sfxDistance < SLIDERRADIUS)
    current_volume = current_volume + (_mouseLoc.x - _prevMouseLoc.x) / 28.0f;
  //set
  current_volume = Clamp(0.0f, 1.0f, current_volume);
  audEngine.SetBusGroupVolume("bus:/SFX", current_volume);
  _sfxVSlider->Translation() = glm::vec3(-6.0f + current_volume * 28,
    TOPY - CHANGEY * SFX_VOLUME, 1);
}

/*!
\brief Update for Back option.
*/
void SettingsMenu::BackUpdate()
{
  if (TheInputManager::Instance()->IsKeyPressed(SDL_SCANCODE_RETURN) ||
    TheInputManager::Instance()->isMousePressed(LEFT) ||
    TheInputManager::Instance()->getButtonState(0, SDL_CONTROLLER_BUTTON_A) ||
    TheInputManager::Instance()->getButtonState(0, SDL_CONTROLLER_BUTTON_B))
  {
    audEngine.PlayEvent(audEngine.GetMusicEvent("MenuStart"));
    // reactivating menu render
    spaceID menu_space = Global::SpaceManager::SpaceUnderneath(_space);
    (**menu_space).GetRenderer().Activate();
    // reactivating menu
    Global::SpaceManager::ResumeBeneath(_space);
    Global::SpaceManager::Remove(_space);
  }
}

// clamp to a range
float SettingsMenu::Clamp(float min, float max, float val)
{
  if (val < min)
    val = min;
  else if (val > max)
    val = max;
  return val;
}