// uses screen.vert as vertex counterpart
#version 330 core

in vec2 texCoord;

uniform float time = 0.0f;
uniform vec2 cameraPosition = vec2(1.0, 1.0);
uniform float frequency = 0.5;
uniform float speedMultiple = 0.5;
uniform vec2 worldSize = vec2(15.0, 15.0);
uniform int layers = 10;
uniform vec4 peakColor = vec4(0.0, 0, 0, 1.0);
uniform vec4 troughColor = vec4(0.0, 0.0, 0, 1.0);

out vec4 color;

float rand(vec2 seed)
{
  float dot_result = dot(seed.xy, vec2(12.9898,72.233));
  return fract(sin(dot_result) * 43758.5453123);
}

float noise(vec2 seed)
{
  vec2 i = floor(seed);
  vec2 f = fract(seed);

  float tl = rand(i);
  float tr = rand(i + vec2(1.0, 0.0));
  float bl = rand(i + vec2(0.0, 1.0));
  float br = rand(i + vec2(1.0, 1.0));

  vec2 t = smoothstep(0.0, 1.0, f);

  return mix(tl, tr, t.x) + (bl - tl) * (1.0 - t.x) * t.y + (br - tr) * t.x * t.y;
}

void main()
{
  vec2 uv = texCoord;
  float light1 = noise(cameraPosition * frequency + uv * worldSize * frequency + time * speedMultiple);
  float light2 = noise(cameraPosition * frequency + uv * worldSize * frequency - time * speedMultiple);
  float light = mix(light1, light2, 0.5);

  light = light * float(layers);
  light = floor(light);
  light = light / float(layers);
  color = mix(peakColor, troughColor, light);
}
