/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderPool.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 13/03/2017
\brief
  Contains the interface for the ShaderPool.
*/
/*****************************************************************************/

#ifndef SHADERPOOL_H
#define SHADERPOOL_H

#include "GraphicShader.h"
#include "ShaderTexture.h"
#include "ShaderScreen.h"
#include "ShaderWater.h"

/*****************************************************************************/
/*!
\class ShaderPool
\brief
  A pool that manages all unique shader types. Each shader type is explicity
  defined.
*/
/*****************************************************************************/
class ShaderPool
{
  public:
    static void Initialize();
    static const GraphicShader & Graphic();
    static const ShaderTexture & Texture();
    static const ShaderTextureSheet & TextureSheet();
    static const ShaderScreen & Screen();
    static const ShaderScreen & ScreenInvert();
    static const ShaderWater & Water();
    static void Purge();
  private:
    //! The default shader for the DefaultGraphic.
    static GraphicShader * _graphicShader;
    //! The default shader for the TextureGraphic.
    static ShaderTexture * _textureShader;
    //! The default shader for a TextureSheet.
    static ShaderTextureSheet * _textureSheetShader;
    //! The default shader for the screen.
    static ShaderScreen * _screenShader;
    //! The shader used for displaying the a fram to the screen with inverted
    // colors.
    static ShaderScreen * _invertScreenShader;
    //! The shader used for rendering the water effect.
    static ShaderWater * _waterShader;
    //! Tracks whether the ShaderPool has been initilized or not.
    static bool _initialized;
    ShaderPool() {}
};

#endif // !SHADERPOOL_H

