/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderTexture.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 07/03/2017
\brief
  Contains the interfaces for Shader types that use Texture sampling.
*/
/*****************************************************************************/

#ifndef SHADERTEXTURE_H
#define SHADERTEXTURE_H

#include "TextureGraphic.h"
#include "Shader.h"

/*****************************************************************************/
/*!
\class ShaderTexture
\brief
  Shader used for displaying a TextureGraphic.
*/
/*****************************************************************************/
class ShaderTexture : public Shader
{
public:
  ShaderTexture(const std::string & vert_file, const std::string & frag_file,
                const TextureGraphic & texture_graphic);
  GLint PositionA() const;
  GLint TexCoordA() const;
  GLint TransformationU() const;
  GLint ColorU() const;
  GLint AdditiveU() const;
  GLint TextureU() const;
private:
  //! The location of the Shader's position attribute.
  GLint _position_A;
  //! The location of the Shader's texCoord attirbute.
  GLint _texCoord_A;
  //! The location of the Shader's transformation uniform.
  GLint _transformation_U;
  //! The location of the Shader's color uniform.
  GLint _color_U;
  //! The location of the Shader's additive uniform.
  GLint _additive_U;
  //! The location of the shader's texture uniform.
  GLint _texture_U;
};

/*****************************************************************************/
/*!
\class ShaderTextureSheet
\brief
  The Shader type that is to be used for displaying the frames on a
  TextureSheet.
*/
/*****************************************************************************/
class ShaderTextureSheet : public ShaderTexture
{
public:
  ShaderTextureSheet(const TextureGraphic & texture_grapic);
  GLint TotalFramesU() const;
  GLint CurrentFrameU() const;
private:
  //! The location of the totalFrames uniform in the animation shader.
  GLint _totalFrames_U;
  //! The location of the currentFrame uniform in the animation shader.
  GLint _currentFrame_U;
};


#endif // !SHADERTEXTURE_H
