/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderWater.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 09/04/2017
*/
/*****************************************************************************/

#include "GraphicPool.h"
#include "OpenGLError.h"

#include "ShaderWater.h"

/*****************************************************************************/
/*!
\brief
  Creates the water shader.
*/
/*****************************************************************************/
ShaderWater::ShaderWater() :
  Shader("Shader/screen.vert", "Shader/water.frag")
{
  // get attrib and uniform locations
  Use();
  _position_A = glGetAttribLocation(ID(), "in_position");
  _texCoord_A = glGetAttribLocation(ID(), "in_texCoord");
  _time_U = glGetUniformLocation(ID(), "time");
  _cameraPosition_U = glGetUniformLocation(ID(), "cameraPosition");
  _frequency_U = glGetUniformLocation(ID(), "frequency");
  _speedMultiple_U = glGetUniformLocation(ID(), "speedMultiple");
  _worldSize_U = glGetUniformLocation(ID(), "worldSize");
  _layers_U = glGetUniformLocation(ID(), "layers");
  _peakColor_U = glGetUniformLocation(ID(), "peakColor");
  _troughColor_U = glGetUniformLocation(ID(), "troughColor");
  // make sure uniforms and attributes were found
  CHECKATTRIBUTE(_position_A, "in_position",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKATTRIBUTE(_texCoord_A, "in_texCoord",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_time_U, "time",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_cameraPosition_U, "cameraPosition",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_frequency_U, "frequency",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_speedMultiple_U, "speedMultiple",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_worldSize_U, "worldSize",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_layers_U, "layers",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_peakColor_U, "peakColor",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_troughColor_U, "troughColor",
    _vertexFile.c_str(), _fragmentFile.c_str());
  //error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderWater.cpp", "ShaderWater()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
  // prep attributes
  glBindVertexArray(GraphicPool::Screen().VertexArray());
  glBindBuffer(GL_ARRAY_BUFFER, GraphicPool::Screen().VertexBuffer());
  glVertexAttribPointer(_position_A, 2, GL_FLOAT,
    GL_FALSE, 4 * sizeof(GLfloat), 0);
  glEnableVertexAttribArray(_position_A);
  glVertexAttribPointer(_texCoord_A, 2, GL_FLOAT, GL_FALSE,
    4 * sizeof(GLfloat), (GLvoid*)(2 * sizeof(GLfloat)));
  glEnableVertexAttribArray(_texCoord_A);
  glBindVertexArray(0);
  glBindBuffer(GL_ARRAY_BUFFER, 0);
  // error check
  error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderWater.cpp", "ShaderWater()");
    error.Code(error_code);
    error.Add("Thrown while preparing the attributes.");
    throw(error);
  }
}

//! /brief Gets the position attribute location. /return In bief.
GLint ShaderWater::PositionA() const       { return _position_A;       }
//! /brief Gets the texCoord attribute location. /return In bief.
GLint ShaderWater::TexCoordA() const       { return _texCoord_A;       }
//! /brief Gets the time uniform location. /return In bief.
GLint ShaderWater::TimeU() const           { return _time_U;           }
//! /brief Gets the cameraPosition uniform location. /return In bief.
GLint ShaderWater::CameraPositionU() const { return _cameraPosition_U; }
//! /brief Gets the frequency uniform location. /return In bief.
GLint ShaderWater::FrequencyU() const      { return _frequency_U;      }
//! /brief Gets the speedMultiple uniform location. /return In bief.
GLint ShaderWater::SpeedMultipleU() const  { return _speedMultiple_U;  }
//! /brief Gets the worldSize uniform location. /return In bief.
GLint ShaderWater::WorldSizeU() const      { return _worldSize_U;      }
//! /brief Gets the layers uniform location. /return In bief.
GLint ShaderWater::LayersU() const         { return _layers_U;         }
//! /brief Gets the peakColor uniform location. /return In bief.
GLint ShaderWater::PeakColorU() const      { return _peakColor_U;      }
//! /brief Gets the troughColor uniform location. /return In bief.
GLint ShaderWater::TroughColorU() const    { return _troughColor_U;    }