/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file SoundEmitter.cpp
\author yourname
\par E-mail: youremail
\par Project: Boat Game
\date 2/17/2017 2:13:04 PM
\brief

\par
Functions Include:
-
*/
/*****************************************************************************/

#include "SoundEmitter.h"
#include "AudioEngine.h"

SoundEmitter::SoundEmitter(const std::map<std::string, std::pair<std::string, bool> > & soundEvents, std::string startingID) 
  : Component("SoundEmitter"), _events(soundEvents), _firstSound(startingID)
{
}

SoundEmitter::~SoundEmitter()
{
  StopAllSounds();
}

/*****************************************************************************/
/*!
\brief
  If first sound is set, plays that sound on startup.
*/
/*****************************************************************************/
void SoundEmitter::Init()
{
  if (!_firstSound.empty())
    PlaySound(_firstSound);
}

/*****************************************************************************/
/*!
\brief
  Plays the sound mapped to the given ID.

\param ID
  The ID for the sound event. Not the event itself! For example, if there is 
  an event :event\SFX\Gun mapped to "gun" then the ID is "gun".
*/
/*****************************************************************************/
void SoundEmitter::PlaySound(std::string ID)
{
  audEngine.PlayEvent(_events[ID].first);
  _events[ID].second = true;
}

/*****************************************************************************/
/*!
\brief
  Stops the sound mapped to the given ID.

\param ID
  The ID for the sound event. Not the event itself! For example, if there is
  an event :event\SFX\Gun mapped to "gun" then the ID is "gun".
*/
/*****************************************************************************/
void SoundEmitter::StopSound(std::string ID)
{
  if (_events[ID].second)
  {
    audEngine.StopEvent(_events[ID].first);
    _events[ID].second = false;
  }
}

/*****************************************************************************/
/*!
\brief
  Stops all sounds controlled by this sound emitter.
*/
/*****************************************************************************/
void SoundEmitter::StopAllSounds()
{
  for each(auto ID in _events)
  {
    StopSound(ID.first);
  }
}

/*****************************************************************************/
/*!
\brief
  Sets a parameter on the event with the given ID.

\param SoundID
  The ID for the sound event. Not the event itself! For example, if there is
  an event :event\SFX\Gun mapped to "gun" then the ID is "gun".

\param SoundParameter
  The string name of the parameter to change.

\param fValue
  The value to set the parameter to.

\return
*/
/*****************************************************************************/
void SoundEmitter::SetSoundParameter(const std::string& SoundID, const std::string& SoundParameter, float fValue)
{
  audEngine.SetEventParameter(_events[SoundID].first, SoundParameter, fValue);
}

Component * CreateSoundEmitterComponent(Json::Value value)
{
  Json::Value soundArray = value["Sounds"];
  std::map<std::string, std::pair<std::string, bool> > soundEvents;
  for (unsigned i = 0; i < soundArray.size(); i++)
  {
    soundEvents[soundArray[i][0].asString()].first = soundArray[i][1].asString();
    soundEvents[soundArray[i][0].asString()].second = false;
  }
  std::string firstSound = value["FirstSound"].asString();
  return new SoundEmitter(soundEvents, firstSound);
}