/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#pragma once
/*****************************************************************************/
/*!
\file SoundEmitter.h
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: Boat Game
\date 2/17/2017 2:13:13 PM
\brief This is the interface for the sound emitter class.
*/
/*****************************************************************************/

#ifndef SoundEmitter_H
#define SoundEmitter_H

#include "Component.h"
#include <map>

/*****************************************************************************/
/*!
\class SoundEmitter
\brief
  The soundemitter is a wrapper around our audio engine. It maps IDs to events
  to make the code for playing sound more readable. It also has some convenience
  functions, like playing a sound on startup.

\par
	Operations include:
	-Play Sound
  -Stop Sound
  -Stop all sounds
  -Set Sound Parameter

\deprecated
	-

\bug
	-
*/
/*****************************************************************************/
class SoundEmitter : public Component
{
public:
  SoundEmitter(const std::map<std::string, std::pair<std::string, bool> > & soundEvents, std::string startingID = std::string());
  ~SoundEmitter();
  void Init();
  void PlaySound(std::string ID);
  void StopSound(std::string ID);
  void StopAllSounds();
  void SetSoundParameter(const std::string& SoundID, const std::string& SoundParameter, float fValue);

private:
  //! Maps event strings to more readable IDs.
  std::map<std::string, std::pair<std::string, bool>> _events;
  std::string _firstSound;
};

Component * CreateSoundEmitterComponent(Json::Value value);
#endif