/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file SpaceManager.h
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 29/11/2016
\brief
  Contains the iterface for the SpaceManager, a static class used to create
  spaces within the engine.
*/
/*****************************************************************************/
#ifndef SPACEMANAGER_H
#define SPACEMANAGER_H

#include "StateFactory.h" // StateFactory
#include "Space.h" // Space
#include "spaceID.h" // spaceID

#include <queue>
#include <list>

namespace Global
{

/*****************************************************************************/
/*!
\class SpaceManager
\brief
  This class is used to add and manage Spaces. Spaces can be added and removed
  in order to render levels on top of eachother.

\par
    Important Notes:
    - The Spaces will be displayed in the order you decied to add them.

\par
    Operations include:
    - Adding a Space
    - Removing a Space
    - Updating all Spaces in the SpaceManager
    - Purging all Spaces in the SpaceManager

\deprecated
    - Spaces used to be stored by value in a list. They are now stored by
      point in a standard list.

\bug
    -
  */
  /*****************************************************************************/
class SpaceManager
 {
  public:
    static spaceID Add(StateType state, float world_height);
    static spaceID Add(std::string fileName, float world_height);
    static spaceID Add(const TransitionInfo & transtion_info);
    static spaceID AddAbove(spaceID space, StateType state, float world_height);
    static spaceID AddBottom(StateType state, float world_height);
    static spaceID AddBottom(const std::string & filename, float world_height);
    static void Remove(spaceID space_index);
    static void RemoveAll();
    static void RemoveAllBeneath(spaceID space_index);
    static bool Update();
    static void PauseAll();
    static void ResumeAll();
    static void RestartAll();
    static void ResumeBeneath(spaceID space);
    static void RestartBeneath(spaceID space);
    static spaceID SpaceUnderneath(spaceID space);
    static State * FindState(StateType state);
    static void Purge();
    static spaceID NullSpaceID();
  private:
    static void RemoveDeadSpaces();
    static void ReorderRenderers();
    //! This is a temporary list of the spaces that need to be removed from the
    // SpaceManager. Since Remove will likely be called during the
    // SpaceManager's update, we do not want to remove any Spaces until after
    // the update since it may cause an error. This means you can remove a Space
    // from within itself.
    static std::queue<spaceID> _deadSpaces;
    //! A list of all the spaces that are within the SpaceManager. This is a
    // list since not that many spaces exist will during runtime, and it makes
    // the removing process easier to manage as it will keep them in order. It 
    // also means that spaceID's will always be acurate since list iterators are
    // not invalidated.
    static std::list<::Space *> _spaces;
    SpaceManager();
};

} // Global

#endif // !SPACEMANAGER_H
