/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Sprite.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 13/03/2017
\brief
  Contains the implementation of the Sprite.
*/
/*****************************************************************************/

#include "Sprite.h"

/*****************************************************************************/
/*!
\brief
  Creates a base sprite. Will start with a pure white color.

\param sprite_type
  The type of the sprite that is being created.
*/
/*****************************************************************************/
Sprite::Sprite(TYPE sprite_type) :
_active(true), _color(1.0f, 1.0f, 1.0f, 1.0f), _type(sprite_type)
{}

/*****************************************************************************/
/*!
\brief
  Overload this to draw a Sprite instance.

\param transformation
  The Object to NDC transformation that will be applied to the sprite
  location.
*/
/*****************************************************************************/
void Sprite::Draw(const glm::mat4 & transformation) const
{}

/*!
\brief Returns the TYPE of the Sprite.
\return The TYPE of the Sprite.
*/
Sprite::TYPE Sprite::SpriteType() const
{
  return _type;
}

/*****************************************************************************/
/*!
\brief
  Change the color that is applied to the Sprite.

\param r
  The red value.
\param g
  The green value.
\param b
  The blue value.
\param a
  The alpha value.
*/
/*****************************************************************************/
void Sprite::SetColor(float r, float g, float b, float a)
{
  _color.r = r;
  _color.g = g;
  _color.b = b;
  _color.a = a;
}

/*****************************************************************************/
/*!
\brief
  Set just the alpha value of the Sprite.

\param a
  The new alpha value.
*/
/*****************************************************************************/
void Sprite::SetAlpha(float a)
{
  _color.a = a;
}