/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Sprite.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 13/03/2017
\brief
  Contains the interface for the Base sprite class.
*/
/*****************************************************************************/

#ifndef SPRITE_H
#define SPRITE_H

#include <GLM/glm/vec4.hpp>
#include <GLM/glm/mat4x4.hpp>

/*****************************************************************************/
/*!
\class Sprite
\brief
  The abstract Sprite. Sprites that derive from this must overload the draw
  function in order to draw the specific sprite type to screen.
*/
/*****************************************************************************/
class Sprite
{
  public:
    //! The possible types of sprites.
    enum TYPE
    {
      GRAPHIC,
      TEXTURE,
      TEXTURESHEET
    };
  public:
    Sprite(TYPE sprite_type);
    virtual void Draw(const glm::mat4 & transformation) const = 0;
    TYPE SpriteType() const;
    void SetColor(float r, float g, float b, float a);
    void SetAlpha(float a);
    //! Tracks whether the Sprite is drawn or not. Check in Renderer.
    bool _active;
  protected:
    //! The type of the Sprite instance.
    TYPE _type;
    //! The color the Sprite is rendered with.
    glm::vec4 _color;

};

#endif // !SPRITE_H