/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file SpritePool.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 16/03/2017
\brief
  Contains the interface of the SpritePool.
*/
/*****************************************************************************/

#ifndef SPRITEPOOL_H
#define SPRITEPOOL_H

#include <vector>

#include "GraphicSprite.h"
#include "TextureSprite.h"
#include "TextureSheetSprite.h"

/*****************************************************************************/
/*!
\class SpritePool
\brief
  Used to store all of the existing Sprites.
*/
/*****************************************************************************/
class SpritePool
{
  public:
    // graphic sprite
    static GraphicSprite * 
    AddGraphic(const GraphicShader * shader);
    // texture sprite
    static TextureSprite * 
    AddTexture(const ShaderTexture * shader, const Texture * texture);
    // texture sheet sprite
    static TextureSheetSprite * 
    AddTextureSheet(const ShaderTextureSheet * shader,
                    const TextureSheet * texture_sheet);
    static void Purge();
  private:
    //! All of the GraphicSpites in the pool.
    static std::vector<GraphicSprite *> _graphicSprites;
    //! All of the TextureSprites in the pool.
    static std::vector<TextureSprite *> _textureSprites;
    //! All of the TextureSheetSprites in the pool.
    static std::vector<TextureSheetSprite *> _textureSheetSprites;
    SpritePool();
};

#endif // !SPRITEPOOL_H
