/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file State.h
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: Boat Game
\date 10/07/2016
\brief
  This is the interface file for the State class.
*/
/*****************************************************************************/

#ifndef STATE_H
#define STATE_H

#include "GameObjectManager.h"
#include "spaceID.h"
#include "StateFactory.h"

/*****************************************************************************/
/*!
\class State
\brief
  This State class is used to house all of the information for a level.
  Within this class, you will find all of the functions needed to load and run
  a level as well as the GameObjectManager that stores and manages all of the
  objects that reside within the state.

\deprecated
  - The ObjectManager used to be global. Now that the object manager is
    instantiated, this class is responsible for holding it's own object manager.

  - This class used to operate with function pointers. These really had no
    purpose since we are using c++. All functions are now virtual functions
    that may or may not be overloaded.
*/
/*****************************************************************************/

// Declaration doesn't need to know what a tilemap is.
class TileMap;

class State
{
    // Only Spaces can run State functions
    friend class Space;
    friend class StateFactory;
  public:
    State();
    GameObjectManager * GetObjectManager();
    const std::vector<TileMap *> & GetTileMaps();
  protected:
    //! This will be used to manage all of the game objects within this state.
    GameObjectManager _objectManager;
    //! This an ID to the Space that this State is being ran in.
    spaceID _space;
    //! Class that manages tile loading
    std::vector<TileMap *> _tileMaps;
  private:
    //! The StateType that is being ran in this State.
    StateType _type;
    void SetType(StateType type);
    void SetSpace(spaceID space);
    StateType Type(void);
    void BaseUpdate();
    void BaseFree();
    virtual void Allocate();
    virtual void Load();
    virtual bool Update();
    virtual void Unload();
    virtual void Free();
    std::string _level;
};

#endif // STATE_H