/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Texture.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Mecha
\date 07/10/2016
\brief
  Contains the interface for a Texture object. To find out more about Texture
  objects, look at the Texture class.
*/
/*****************************************************************************/

#ifndef TEXTURE_H
#define TEXTURE_H

#include <GL\glew.h>
#include <string>

/*****************************************************************************/
/*!
\class Texture
\brief
  A texture that can be bound before a TextureGraphic is drawn in order to
  display the texture on top of the TextureGraphic.
  
\par Important Notes
  - Texture's will not work on a Graphic. It must be a TextureGraphic.
*/
/*****************************************************************************/
class Texture
{
  public:
    Texture(const std::string & image_file);
    void Purge();
    void Bind() const;
    int Width() const;
    int Height() const;
    float AspectRatio() const;
    const std::string & FileName() const;
  protected:
    //! The OpenGL texture ID for binding the texture.
    GLuint _texture;
    //! The aspect ratio of the Texture.
    float _aspectRatio;
    //! The width of the texture in pixels.
    int _width;
    //! The height of the texture in pixels.
    int _height;
    //! The name of the file that the texture was loaded from.
    std::string _textureFile;
};

#endif // !TEXTURE_H
