/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureGraphic.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 26/12/2017
\brief
  Contains the implementation of the TextureGraphic. Look at the TextureGraphic
  class for more information.
*/
/*****************************************************************************/

#include "OpenGLError.h"

#include "TextureGraphic.h"

/*****************************************************************************/
/*!
\brief
  Creates a TextureGraphic by setting up all of the OpenGL buffers needed for
  a TextureGraphic.
*/
/*****************************************************************************/
TextureGraphic::TextureGraphic() : Graphic()
{
  // setting up OpenGL buffers.
  GLfloat vertices[20] = { -0.5f,  0.5f, 0.0f,   0.0f, 1.0f,
                            0.5f,  0.5f, 0.0f,   1.0f, 1.0f,
                           -0.5f, -0.5f, 0.0f,   0.0f, 0.0f,
                            0.5f, -0.5f, 0.0f,   1.0f, 0.0f };
  GLuint indices[6] = { 0, 1, 2, 1, 2, 3};
  PrepBuffer(vertices, 20, indices, 6);
}

/*****************************************************************************/
/*!
\brief
  Draws that TextureGraphic to the screen using the shader currently in use.
*/
/*****************************************************************************/
void TextureGraphic::Draw() const
{
  // draw
  glBindVertexArray(_vertexArray);
  glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
  glBindVertexArray(0);
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureGraphic.cpp", "Draw");
    error.Code(error_code);
    throw(error);
  }
}
