/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TexturePool.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 14/03/2017
\brief
  Contains the interface for the TexturePool.
*/
/*****************************************************************************/

#ifndef TEXTUREPOOL_H
#define TEXTUREPOOL_H

#include <string>
#include <map>

#include "Texture.h"
#include "TextureSheet.h"

/*****************************************************************************/
/*!
\class TexturePool
\brief
  Used for storing all Textures that are currently in use.
*/
/*****************************************************************************/
class TexturePool
{
  public:
    static Texture * Add(const std::string & filename);
    static TextureSheet * AddSheet(const std::string & filename,
                                   unsigned int total_frames,
                                   unsigned int rows, unsigned int columns);
    static void Purge();
  private:
    //! All of the textures that have been added to the TexturePool.
    static std::map<std::string, Texture *> _pool;
    //! All of the texture sheets that have been added to the pool.
    static std::map<std::string, TextureSheet *> _sheetPool;
    TexturePool() {}
};

#endif // !TEXTUREPOOL_H
