/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureSheet.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 07/03/2017
*/
/*****************************************************************************/
#ifndef TEXTURESHEET_H
#define TEXTURESHEET_H

#include <string>

#include "Texture.h"

/*****************************************************************************/
/*!
\class TextureSheet
\brief
  Used to split a Texture into multiple rows and columns of textures that can
  be used for animations and tiling.
*/
/*****************************************************************************/
class TextureSheet : public Texture
{
  public:
    TextureSheet(const std::string & file, unsigned int total_frames,
                 unsigned int columns, unsigned int rows);
    unsigned int TotalFrames() const;
    unsigned int Rows() const;
    unsigned int Columns() const;
    float FrameAspectRatio() const;
   private:
     //! The total number of frames on the TextureSheet.
     unsigned int _totalFrames;
     //! The total number of columns on the TextureSheet.
     unsigned int _columns;
     //! The total number of rows on the TextureSheet.
     unsigned int _rows;
     //! The aspect ratio (width / height) for a single frame.
     float _frameAspectRation;
};

#endif // !TEXTURESHEET_H

