/*****************************************************************************/
/*!
\file TextureSheetShader.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 08/03/2017
\brief
  Contains the implmentation for the TextureSheetShader.
*/
/*****************************************************************************/

#include "OpenGLError.h"

#include "TextureSheetShader.h"

//! The vertex file used for this Shader.
#define VERT_FILE "Shader/TextureSheet.vert"
//! The fragment file used for this Shader.
#define FRAG_FILE "Shader/TextureSheet.frag"

/*****************************************************************************/
/*!
\brief
  Creates a TextureSheetShader and finds the uniforms that should be in the
  TextureSheetShader.

\param texture_graphic
  The TextureGraphic that this shader will display.
*/
/*****************************************************************************/
TextureSheetShader::TextureSheetShader(const TextureGraphic & texture_graphic) :
  TextureGraphicShader(VERT_FILE, FRAG_FILE, texture_graphic)
{
  // finding uniforms
  _totalFrames_U = glGetUniformLocation(ID(), "totalFrames");
  _currentFrame_U = glGetUniformLocation(ID(), "currentFrame");
  // checking that uniforms were found
  CHECKUNIFORM(_totalFrames_U, "totalFrames", VERT_FILE, FRAG_FILE);
  CHECKUNIFORM(_currentFrame_U, "currentFrame", VERT_FILE, FRAG_FILE);
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureGraphicShader.cpp", "TextureGraphicShader()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
}

/*!
\brief Gets the location of the totalFrames uniform.
\return The location of the totalFrames uniform.
*/
GLint TextureSheetShader::TotalFramesU() const
{
  return _totalFrames_U;
}

/*!
\brief Gets the location of the currentFrame uniform.
\return The location of the currentFrame uniform.
*/
GLint TextureSheetShader::CurrentFrameU() const
{
  return _currentFrame_U;
}