/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureSheetSprite.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 09/03/2017
\brief
  Contains the interface for the TextureSheetSprite.
*/
/*****************************************************************************/

#ifndef TEXTURESHEETSPRITE_H
#define TEXTURESHEETSPRITE_H

#include <GLM/glm/vec2.hpp>

#include "Sprite.h"
#include "ShaderTexture.h"
#include "TextureSheet.h"

/*****************************************************************************/
/*!
\class TextureSheetSprite
\brief
  The Sprite type that is used to display frames on a TextureSheet. This Sprite
  will be drawn on a TextureGraphic.
*/
/*****************************************************************************/
class TextureSheetSprite : public Sprite
{
  public:
    TextureSheetSprite(const ShaderTextureSheet * shader,
                       const TextureSheet * texture_sheet);
    virtual void Draw(const glm::mat4 & object_to_ndc) const;
    void NextFrame();
    void PreviousFrame();
    void SetCurrentFrame(unsigned int column, unsigned int row);
    void SetFrameNumber(unsigned int new_current_frame);
    void SetAdditive(bool is_additive);
    const TextureSheet & Sheet();
  private:
    //! The shader used to draw the sprite.
    const ShaderTextureSheet * _shader;
    //! The texture sheet whose frames will be displayed.
    const TextureSheet * _textureSheet;
    //! The location of the current frame on the TextureSheet.
    glm::tvec2<unsigned int> _currentFrame;
    //! The frame number of the current frame (starts at 0).
    unsigned int _currentFrameNumber;
    //! Determines if the sprite color adds to the texture color or multiplies.
    bool _isAdditive;
};

#endif // !TEXTURESHEETSPRITE_H

