/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TileMap.cpp
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: 
\date DD/MM/YYYY
\brief


\par
    Functions Include:
    -
*/
/*****************************************************************************/
#include "TileMap.h"
#include <json\json.h>
#include <fstream>
#include <iostream>
#include "Error.h"
#include "GameObjectManager.h"
#include "Sprite.h"

#include "Physics.h"
#include "BoxCollider.h"
#include "Camera.h"
#include "ShaderPool.h"
#include "TexturePool.h"
#include "SpritePool.h"

TileMap::TileMap(Json::Value value, GameObjectManager * objectManager, const TileData & data) : _objectManager(objectManager), _tileData(data)
{
  _offset.x = value["x"].asFloat();
  _offset.y = value["y"].asFloat();
  _width = data.Scale;

  _tileData.tile_archetype = objectManager->CreateObject("Tile");
  _tileData.tile_archetype->Add(new Physics(0, Physics::Static));
  _tileData.tile_archetype->Add(new BoxCollider(0, 0, "Tile"));
  _tileData.base_pos = _offset;

  _tileData.texture_sheet = SpritePool::AddTextureSheet(&ShaderPool::TextureSheet(), TexturePool::AddSheet(_tileData.spritesheet, _tileData.tile_count, _tileData.rows, _tileData.columns));


  /*
  for each(std::string name in value["data"].getMemberNames())
  {
    TileData data;
    //data.archetype = value["data"][name]["archetype"].asString();
    data.collidable = value["data"][name]["collision"].asBool();
    data.archetypeObject = objectManager->CreateArchetypeObject(value["data"][name]["archetype"].asString());
    data.archetypeObject->Add(new Physics(0, Physics::Static));
    data.archetypeObject->Add(new BoxCollider(0, 0, "Tile"));
    _tileData.push_back(data);
  }
  */
  /*
  for (int i = 0; i < 100; i++)
  {
    if (i == 0)
    {
      TileData data;
      data.collidable = false;
      data.archetypeObject = NULL;
      _tileData.push_back(data);
    }
    else if (i != 39)
    {
      TileData data;
      data.collidable = true;
      data.archetypeObject = objectManager->CreateArchetypeObject("Tile");
      data.archetypeObject->Add(new Physics(0, Physics::Static));
      data.archetypeObject->Add(new BoxCollider(0, 0, "Tile"));
      _tileData.push_back(data);
    }
    else
    {
      TileData data;
      data.collidable = false;
      data.archetypeObject = objectManager->CreateArchetypeObject("waterTile");
      data.archetypeObject->Add(new Physics(0, Physics::Static));
      data.archetypeObject->Add(new BoxCollider(0, 0, "Tile"));
      _tileData.push_back(data);
    }
  */

  for (unsigned int i = 0; i < value["height"].asUInt(); i++)
  {
    _tileMap.push_back(std::vector<int>());
  }

  Json::Value Array = value["data"];

  for (Json::ArrayIndex i = 0; i < value["height"].asUInt(); i++)
  {
    for (Json::ArrayIndex j = 0; j <  value["width"].asUInt(); j++)
    {
      int index = i * value["width"].asInt() + j % value["width"].asInt();
      int value = Array[index].asInt();
      _tileMap[i].push_back(value);
      //int index = Array[i][j].asInt();
      //Object * obj = _objectManager->CreateArchetypeObjectAtPosition(_tileData[index].archetype, glm::vec3(j * _width + _offset.x, -signed(i) * _width + _offset.y, 0));
    }
  }
  _tileData._tileMap = &_tileMap;
}

int TileMap::TileValueAtPosition(glm::vec2 position)
{
  auto index = Index(position);
  if (index.first < 0 || static_cast<unsigned>(index.first) >= _tileMap[0].size() 
    || index.second < 0 || static_cast<unsigned>(index.second) >= _tileMap.size())
    return 0;
  return _tileMap[index.second][index.first];
}


void TileMap::Update()
{
  /*
  for (auto tileMapItr = _tileMaps.rbegin(); tileMapItr != _tileMaps.rend(); tileMapItr++)
  {
    std::vector<std::vector<int>> & tileMap = tileMapItr->second;
    for (size_t i = 0; i < tileMap.size(); i++)
    {
      for (size_t j = 0; j < tileMap[i].size(); j++)
      {
        int index = tileMap[i][j];
        Object * obj = _tileData[index].archetypeObject;
        if (obj != NULL)
        {
          // PROBLEM HERE. WE NEED THE TILEMAP THE REGISTER WITH THE
          // RENDERER SOMEHOW. TILEMAPS WILL NOT WORK LIKE THIS ANYMORE
          glm::vec3 new_translation(j * _width + _offset.x, -signed(i) * _width + _offset.y, 0);
          obj->GetTransform()->SetTranslation(glm::vec3(new_translation));
          obj->Update();          
        }
      }
    }
  }
  */
}

glm::vec2 TileMap::TileCenterAtPosition(glm::vec2 position)
{
  auto index = Index(position);
  return glm::vec2(index.first, -index.second) * _width + _offset;
}

std::pair<int, int> TileMap::Index(glm::vec2 position) 
{
  position -= _offset;
  position.y *= -1;
  int x = static_cast<int>((position.x + _width / 2) / _width);
  int y = static_cast<int>((position.y + _width / 2) / _width);
  return std::pair<int, int>(x, y);
}

float TileMap::GetWidth() const
{
  return _width;
}

const TileData * TileMap::GetTileData() const
{
  return &_tileData;
}