/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#pragma once
/*****************************************************************************/
/*!
\file TileMap.h
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: X-Craft
\date 01/20/2017
\brief 
  This is the interface for the tilemap, which stores tile data 
  and renders the tiles on the screen.

*/
/*****************************************************************************/

#include <map>
#include <string>
#include <vector>
#include <glm\glm.hpp>

#include <json\json.h>

#include "TextureSheetSprite.h"

class Object;

struct TileData
{
  unsigned rows;       // Number of rows in the tile sheet
  unsigned columns;    // Number of columns in the tile sheet 
  unsigned tile_count; // Number of tiles in the tile sheet
  float Scale;      // Scale of each tile
  glm::vec2 base_pos;
  TextureSheetSprite * texture_sheet;
  std::string spritesheet; // File path of the sprite sheet.

  bool collidable;
  Object * tile_archetype;
  // Pointer to the tilemaps. Don't worry about what they're called
  // just draw all of them. 
  std::vector<std::vector<int>> * _tileMap;
};

class GameObjectManager;

/*****************************************************************************/
/*!
\class TileMap
\brief
  

\par
    Operations include:
    -

\deprecated
    -

\bug
    -
*/
/*****************************************************************************/
class TileMap
{
public:
  TileMap(Json::Value value, GameObjectManager * objectManager, const TileData & data);
  int TileValueAtPosition(glm::vec2 position);
  glm::vec2 TileCenterAtPosition(glm::vec2 position);
  float GetWidth() const;
  const TileData * GetTileData() const;
  
  void Update();

private:
  // ! Map of 2-dimensional arrays with tile data
  std::vector<std::vector<int>> _tileMap;
  // ! int is the integer index of the tile, tile data has information about the tile.
  TileData _tileData;
  glm::vec2 _offset;
  float _width;
  GameObjectManager * _objectManager;
  std::pair<int, int> Index(glm::vec2 position);
};