/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Time.cpp
\author Zach Dolph / Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 13/01/2017
\brief
  Contains the implementation of the Time class. Look in the class header fie
  for more information.
*/
/*****************************************************************************/
#include <SDL\SDL_timer.h>

#include "Time.h"

// static initializations
float Time::_deltaTime = 0.0f;
float Time::_totalTime = 0.0f;
int Time::_deltaTicks = 0;
int Time::_ticks = 0;
int Time::_ticksPrev = 0;

/*****************************************************************************/
/*!
\brief
  Updates all of the values within the Time class. This should be called at the
  start of every frame so all other functionality is based off of the correct
  times.
*/
/*****************************************************************************/
void Time::Update()
{
  _ticks = SDL_GetTicks();
  _deltaTicks = _ticks - _ticksPrev;
  _deltaTime = (float)_deltaTicks / (float)1000;
  _totalTime += _deltaTime;
  _ticksPrev = _ticks;
}

/*****************************************************************************/
/*!
\brief
  Gets the current value of _deltaTime.

\return The value of _deltaTime.
*/
/*****************************************************************************/
float Time::DT()
{
  return _deltaTime;
}

/*****************************************************************************/
/*!
\brief
  Returns the total time (in seconds) that has passed since the program began
  running.

\return The current value of _totalTime.
*/
/*****************************************************************************/
float Time::TotalTime()
{
  return _totalTime;
}
