/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Time.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 13/01/2017
\brief
  Contains the interface for the static Time class. A class used to track time
  values that are associated with the running program.
*/
/*****************************************************************************/
#ifndef TIME_H
#define TIME_H

/*****************************************************************************/
/*!
\class Time
\brief
  This static class is responsible for managing the values associated with frame
  times and program ticks. It will store the amount of time passed since it was
  previously updated and other important information related to program time.

\par Important Notes
  - Update this class at the very start of each frame so all other functionality
    uses the proper time values.
*/
/*****************************************************************************/
class Time
{
  public:
    static void Update();
    static float DT();
    static float TotalTime();
  private:
    // static class
    Time() {}
    //! The difference of time (in seconds) from the Time classes Update before
    // the most recent Update to the most recent Update.
    //\par Diagram
    //  - (Previous Update) ---------> (Most Recent Update)
    //  - The arrow represents _deltaTime in seconds.
    static float _deltaTime;
    //! The total time that has passed (in seconds) since the program was
    // launched.
    static float _totalTime;
    //! The same as _deltaTime, but this is the number of Ticks passed rather
    // than the number of seconds
    static int _deltaTicks;
    //! The total number of ticks the program has experienced since the program
    // started running.
    static int _ticks;
    //! The total number of ticks that had been experienced until the previous
    // update. The differnce between _ticks and _ticksPrev is the value of
    // _deltaTicks.
    static int _ticksPrev;
};

#endif // TIME_H
