/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Transition.h
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 15/04/2017
\brief
  The Transition State.
*/
/*****************************************************************************/

#include <string>

#include "GraphicSprite.h"
#include "State.h"

/*****************************************************************************/
/*!
\class Transition
\brief
  This state is used to create a smooth transition from one state to another.
*/
/*****************************************************************************/
class Transition : public State
{
public:
  Transition(const TransitionInfo & transition_info);
  virtual void Allocate();
  virtual bool Update();
  virtual void Free();
protected:
  //! The current state that the transition is in.
  enum TRANSTITION_STATE
  {
    FADEIN,
    STILL,
    FADEOUT
  };
  //! Contains information about the transition.
  TransitionInfo _info;
  //!
  float _startVolume;
  //! Used to track the time passed to make the transition seemless.
  float _currentTime;
  //! The current state that the transition is in.
  TRANSTITION_STATE _currentState;
  //! The sprite that covers the screen to create the transition effect.
  GraphicSprite * _coverSprite;
  //! The transform of the sprite.
  Transform _coverT;
  //! The sprite that displays a loading symbol
  Sprite * _loadingSprite;
  //! Tracks whether the level being transitioned to has loaded.
  bool _levelLoaded;

};
