/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#include "Vec2Math.h"

#include "Object.h"
#include "GameObjectManager.h"
#include "Turret.h"
#include "Time.h"
#include "Physics.h"
#include "AudioEngine.h"
#include "EnemyHealth.h"

#define ACTIVATEDISTANCE 20.0f

Turret::Turret(float fire_rate, float bullet_speed, float activation_range) :
  Component("Turret"), _fireConstant(1.0f / fire_rate), _bulletSpeed(bullet_speed),
  _activationRange(activation_range), _timePassed(0.0f)
{}

Turret::~Turret()
{}

void Turret::Init()
{
  _player = Owner().ObjectManager()->FindObjectByName("Player"); 
  _sprite = Owner().GetSprite();
  Owner().SubscribeCallback(EnemyDamaged::DeathID, std::bind(&Turret::Dead, this, std::placeholders::_1));
}

void Turret::Dead(Event * event)
{
  Owner().ObjectManager()->CreateArchetypeObjectAtPosition("explosion", Owner().GetTransform()->GetTranslation());
  audEngine.PlayEvent("event:/SFX/enemyexplosion");
}

void Turret::Update()
{
  if (!_player)
    return;
  glm::vec2 player_tran(_player->GetTransform()->GetAbsoluteTranslation());
  glm::vec2 turret_tran(Owner().GetTransform()->GetAbsoluteTranslation());
  float distance = glm::distance(player_tran, turret_tran);
  if (distance < _activationRange) {
    _timePassed += Time::DT();
    // set active color
    float red_color = _timePassed / _fireConstant;
    _sprite->SetColor(red_color, 0.3f, 0.3f, 1.0f);
    // set correct angle
    glm::vec2 facing(player_tran - turret_tran);
    float angle = Math::AngleBetweenSlow(facing, glm::vec2(1.0f, 0.0f));
    angle *= glm::sign(glm::dot(facing, glm::vec2(0, 1)));
    Owner().GetTransform()->SetRotation(angle);
    // fire
    if (_timePassed > _fireConstant) {
      Shoot(facing);
      _timePassed = 0.0f;
    }
  }
  else {
    _sprite->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
    _timePassed = 0.0f;
  }

}

void Turret::Shoot(const glm::vec2 & direction)
{
  glm::vec2 norm_direction = glm::normalize(direction);
  Object * bullet = Owner().ObjectManager()->CreateArchetypeObjectAtPosition("turretBullet", Owner().GetTransform()->GetTranslation() + glm::vec3(norm_direction, 1.0f) * 2.5f);
  Physics * phy_comp = bullet->Find<Physics>();
  phy_comp->SetVelocity(norm_direction * _bulletSpeed);
  audEngine.PlayEvent("event:/SFX/TurretFire");
}

Component * CreateTurretComponent(Json::Value value)
{
  return new Turret(value["fireRate"].asFloat(), value["bulletSpeed"].asFloat(), value["activationRange"].asFloat());
}