/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#include "Vec2Math.h"

namespace Math
{
	/*****************************************************************************/
	/*!
	\brief
		Gets the square length of a vector

	\param v
		The vector to get the length of.

	\return
		Returns the square length
	*/
	/*****************************************************************************/
	float LengthSquared(glm::vec2 v)
	{
		return v.x * v.x + v.y * v.y;
	}

	/*****************************************************************************/
	/*!
	\brief
		Returns the square distance between two point vectors.

	\param v
		The first point

	\param w
		The second point

	\return
		Returns the square length
	*/
	/*****************************************************************************/
	float DistanceSquared(glm::vec2 v, glm::vec2 w)
	{
		float x = w.x - v.x;
		float y = w.y - v.y;
		return x*x + y*y;
	}

  /*****************************************************************************/
  /*!
  \brief
    Calculates the angle between two vectors. Use this if you cannot garuntee
    that the vectors are going to be normalized ahead of time.

  \param v
    An un-normalized 2D vector

  \param w
    An un-normalized 2D vector

  \return
    Returns the angle in radians.
  */
  /*****************************************************************************/
  float AngleBetweenSlow(glm::vec2 v, glm::vec2 w)
  {
    float lengthV = glm::length(v);
    float lengthW = glm::length(w);
    if (lengthV == 0 || lengthW == 0)
      return 0;
    return glm::acos(glm::dot(v / lengthV, w / lengthW));
  }

  /*****************************************************************************/
  /*!
  \brief
    Calculates the angle between two vectors. Use this ONLY if you can garuntee that
    the vectors have been normalized.

  \param v
    A normalized 2D vector

  \param w
    A normalized 2D vector

  \return
    Returns the angle in radians.
  */
  /*****************************************************************************/
  float AngleBetweenFast(glm::vec2 v, glm::vec2 w)
  {
    return glm::acos(glm::dot(v, w));
  }

  glm::vec2 RotateVector(glm::vec2 v, float angle)
  {
    return glm::vec2(v.x * glm::cos(angle) - v.y * glm::sin(angle),
      v.x * glm::sin(angle) + v.y * glm::cos(angle));
  }
}