/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
#include <iostream> 

#include "SpaceManager.h"
#include "StateFactory.h"
#include "Context.h"
#include "Graphics.h"
#include "CollisionTable.h"
#include "InputManager.h"
#include "ArchetypeManager.h"
#include "SplashScreen.h"
#include "AudioEngine.h"
#include "Time.h"
#include "GraphicPool.h"
#include "ShaderPool.h"
#include "TexturePool.h"
#include "RendererPool.h"

#include <time.h>

int main(int argc, char ** argv)
{

  srand((unsigned)time);

#if RELEASE
  Context::Create("X-Craft", 0, 0);
#endif // !RELEASE

#if _DEBUG
  Context::Create("X-Craft", 1280, 720);
#endif // !_DEBUG
  Graphics::BlendMode();

  GraphicPool::Initialize();
  ShaderPool::Initialize();
  RendererPool::Initialize();

  CollisionTable::InitCollisionTables();
  archetypeManager.InitArchetypes();

  audEngine.Init();
  audEngine.InitBanks();

  bool game_active = true;
  bool pause = false;

#if RELEASE
  Global::SpaceManager::Add(SPLASH_SCREEN, 30);
#endif // RELEASE

#if _DEBUG
  Global::SpaceManager::Add(MENU_MAIN, 30);
#endif // _DEBUG

  while (Context::KeepOpen() && game_active)
  {
    Time::Update();
    InputManager::Instance()->update();
    game_active = Global::SpaceManager::Update();
    audEngine.Update();


    try {
      RendererPool::Render();
    }
    catch (const Error & error) {
      std::cout << error;
    }

    Context::Swap();
    Context::Clear(0.0f, 0.0f, 0.0f);
  }
  Global::SpaceManager::Purge();
  
  RendererPool::Purge();
  TexturePool::Purge();
  ShaderPool::Purge();
  GraphicPool::Purge();
  Context::Purge();
  return 0;
}